/**
 * @file city-banners.ts
 * @copyright 2021-2025, Firaxis Games
 * @description City Banners' logic for the indiviudal banners
 */
import CityBannerManager from '/base-standard/ui/city-banners/city-banner-manager.js';
import DiplomacyManager from '/base-standard/ui/diplomacy/diplomacy-manager.js';
import { BannerType, CityStatusType, BANNER_INVALID_LOCATION } from '/base-standard/ui/city-banners/banner-support.js';
import { Icon } from '/core/ui/utilities/utilities-image.js';
import { MustGetElement } from '/core/ui/utilities/utilities-dom.js';
import { ComponentID } from '/core/ui/utilities/utilities-component-id.js';
import { CreateElementTable } from '/core/ui/utilities/utilities-dom.js';
import FxsActivatable from '/core/ui/components/fxs-activatable.js';
// ------------------ MODIFIED BY NASU ------------------ START ------------------
import { DebugUtils } from '/scripts/debug-utils.js';
import { animationClock } from '/scripts/war-icon-sync-clock.js';
import { getSettings } from '/scripts/nasuellia-alt-unit-flags-and-bars-settings.js';
const getCurrentSettings = () => getSettings("nasuelliaAltUnitFlagsAndBarsSettingsKey");
const BANNER_ANCHOR_OFFSET = { x: 0, y: 0, z: 48 };
const hasPulseGlow = document.querySelector('style[data-pulse-glow]');
if (!hasPulseGlow) {
    const styleSheet = document.createElement('style');
    styleSheet.setAttribute('data-pulse-glow', 'true');
    styleSheet.textContent = `@keyframes pulseGlow {
    0% { transform: scale(1.025, 1.025); opacity: 0.5; }
    50% { transform: scale(1.05, 1.25); opacity: 1; }
    100% { transform: scale(1.025, 1.025); opacity: 0.5; }
}`;
    document.head.appendChild(styleSheet);
}

function resetChildrenStyles(parentElement, propertyNames) {
    Array.from(parentElement.children).forEach(child => {
        propertyNames.forEach(prop => {
            switch(prop) {
                case 'position':
                    child.style.position = 'static';
                    break;
                case 'top':
                    child.style.top = 'auto';
                    break;
                case 'marginTop':
                    child.style.marginTop = '0';
                    break;
                default:
                    child.style[prop] = '';
            }
        });
    });
}

// ------------------ MODIFIED BY END ------------------ START ------------------
export class CityBannerComponent extends FxsActivatable {
    constructor() {
        super(...arguments);
        this._worldAnchorHandle = null;
        this.inputSelector = ".city-banner__container, .city-banner__queue-container, .city-banner__portrait";
        this.componentID = ComponentID.getInvalidID();
        this.isHidden = false;
        this.location = { x: 0, y: 0 };
        this.city = null;
        this.updateBuildsQueued = false;
        this.updateNameQueued = false;
        this.onActivateEventListener = this.onActivate.bind(this);
        this.elements = CreateElementTable(this.Root, {
            capitalIndicator: '.city-banner__capital-star',
            cityStateColor: '.city-banner__city-state-type',
            container: '.city-banner__container',
            growthQueueContainer: '.city-banner__population-container',
            growthQueueMeter: '.city-banner__population-ring',
            growthQueueTurns: '.city-banner__population-container > .city-banner__turn > .city-banner__turn-number',
            productionQueue: '.city-banner__queue-container.queue-production',
            productionQueueMeter: '.city-banner__production-ring',
            productionQueueIcon: '.city-banner__queue-img',
            productionQueueTurns: '.city-banner__queue-container.queue-production > .city-banner__turn > .city-banner__turn-number',
            portrait: '.city-banner__portrait',
            portraitIcon: '.city-banner__portrait-img',
            urbanReligionSymbol: '.city-banner__religion-symbol',
            ruralReligionSymbol: '.city-banner__religion-symbol.religion-symbol--right',
            urbanReligionSymbolBackground: '.city-banner__religion-symbol-bg',
            ruralReligionSymbolBackground: '.city-banner__religion-symbol-bg.religion-bg--right',
            statusContainer: '.city-banner__status',
            statusIcon: '.city-banner__status-icon',
            statusBg: '.city-banner__status-background',
            portraitImg: '.city-banner__portrait-img',
            cityNameContainer: '.city-banner__name-container',
            cityName: '.city-banner__name',
            popCount: '.city-banner__population-number',
            civPatternContainer: '.city-banner__pattern-container',
            civPattern: '.city-banner__pattern',
            unrestTurns: '.city-banner__unrest > .city-banner__time-container > .city-banner__time-text',
            razedTurns: '.city-banner__razing > .city-banner__time-container > .city-banner__time-text',
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            growthQueueContainer: '.city-banner__population-container',
            productionQueueContainer: '.city-banner__queue-container',
            capitalStar: '.city-banner__capital-star',
            urbanReligion: '.city-banner__religion-symbol',
            ruralReligion: '.religion-symbol--right',
            popRing: '.city-banner__population-ring',
            prodRing: '.city-banner__production-ring',
            prodIcon: '.city-banner__queue-img',
            prodTurns: '.city-banner__queue-container > .city-banner__turn-number',
            cityStateBorder: '.city-banner__city-state-border',
            cityStateRing: '.city-banner__city-state-ring',
            outerContainer: '.city-banner__stretch-bg',
            cityStateIcon: '.city-banner__city-state-icon',
            conqueredIcon: '.city-banner__conquered-icon',
            unrestIcon: '.city-banner__unrest-icon',
            razingIcon: '.city-banner__razing-icon',
            portraitBgHex: '.city-banner__portrait-bg1',
            portraitBgFrame: '.city-banner__portrait-bg2',
            razingTurns: '.city-banner__razing .city-banner__time-text',
            vSlot: '.city-banner__name-vslot',
            hSlot1: '.city-banner__name-vslot > fxs-hslot:nth-child(1)',
            hSlot2: '.city-banner__name-vslot > .city-banner__status-religion',
            religionSlot: '.city-banner__name-vslot > .city-banner__status-religion > .city-banner__religion',
            unrestTimeContainer: '.city-banner__unrest > .city-banner__time-container',
            razingTimeContainer: '.city-banner__razing > .city-banner__time-container'
        });
        this.iconSwapSubscription = null;
        this.originalBannerIcon = null;
        this.urbanReligionAnimationLoopTimer = null;
        this.urbanReligionAnimationTracker = false;
        this.ruralReligionAnimationLoopTimer = null;
        this.ruralReligionAnimationTracker = false;
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.civSymbols = null;
        this.civPatternElements = null;
    }
    get bannerLocation() {
        return this.location;
    }
    queueBuildsUpdate() {
        if (this.updateBuildsQueued)
            return;
        this.updateBuildsQueued = true;
        requestAnimationFrame(this.doBuildsUpdate.bind(this));
    }
    doBuildsUpdate() {
        this.realizeBuilds();
        this.realizeHappiness();
        this.realizePopulation();
        this.realizeReligion();
        this.updateBuildsQueued = false;
    }
    queueNameUpdate() {
        if (this.updateNameQueued)
            return;
        this.updateNameQueued = true;
        requestAnimationFrame(this.doNameUpdate.bind(this));
    }
    // TODO: I don't believe this should be rebuilding the entire banner?
    doNameUpdate() {
        this.buildBanner();
        // All other queues cleared since full banner is rebuilt.
        this.updateBuildsQueued = false;
        this.updateNameQueued = false;
    }
    getDebugString() {
        return `'${this.location.x},${this.location.y}' for ${ComponentID.toLogString(this.componentID)}`;
    }
    getKey() {
        return ComponentID.toBitfield(this.componentID);
    }
    getLocation() {
        return this.location;
    }
    /**
     * getDebugLocation parses the data-debug-plot-index attribute on the DOM element and returns a float2 if it is valid.
     * This is used for generating many city banners attached to one city, but in differing plots, for stress testing.
     */
    getDebugLocation() {
        const plotIndexAttr = this.Root.getAttribute('data-debug-plot-index');
        if (!plotIndexAttr) {
            return null;
        }
        const plotIndex = parseFloat(plotIndexAttr);
        if (isNaN(plotIndex)) {
            return null;
        }
        return GameplayMap.getLocationFromIndex(plotIndex);
    }
    onAttach() {
        super.onAttach();
        this.Root.classList.add("-top-9", "absolute", "flex", "flex-row", "justify-start", "items-center", "flex-nowrap", "bg-center", "whitespace-nowrap", "bg-no-repeat");
        engine.on('BeforeUnload', this.onUnload, this);
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        engine.on('DiplomacyEventStarted', this.onDiplomacyEventStarted, this);
        engine.on('DiplomacyEventEnded', this.onDiplomacyEventEnded, this);
        engine.on('TurnBegin', this.onTurnBegin, this);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.Root.addEventListener('action-activate', this.onActivateEventListener);
        const attrComponentID = this.Root.getAttribute('city-id');
        const attrX = this.Root.getAttribute('x');
        const attrY = this.Root.getAttribute('y');
        this.componentID = ComponentID.fromString(attrComponentID);
        this.location = this.getDebugLocation() ?? { x: BANNER_INVALID_LOCATION, y: BANNER_INVALID_LOCATION };
        if (attrX !== null && attrY !== null) {
            const x = Number.parseInt(attrX);
            const y = Number.parseInt(attrY);
            if (!isNaN(x) && !isNaN(y)) {
                this.location.x = x;
                this.location.y = y;
            }
        }
        if (ComponentID.isInvalid(this.componentID)) {
            console.error("City banner could not attach to manager because componentID sent was invalid.");
            return;
        }
        const manager = CityBannerManager.instance;
        manager.addChildForTracking(this);
        this.city = Cities.get(this.componentID);
        // if the banner manager doesn't know where we go, try to find out ourselves
        if (this.location.x == BANNER_INVALID_LOCATION || this.location.y == BANNER_INVALID_LOCATION) {
            if (this.city) {
                this.location.x = this.city.location.x;
                this.location.y = this.city.location.y;
            }
            else {
                console.error(`city-banners: Got placeholder location for non-city ${ComponentID.toLogString(this.componentID)}`);
            }
        }
        this.makeWorldAnchor(this.location);
        // Show or hide based on the fog-of-war (FOW)
        this.setVisibility(this.getVisibility());
        this.buildBanner();
    }
    /** Debug only: (this part of the) DOM is reloading. */
    onUnload() {
        this.cleanup();
    }
    onDetach() {
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        engine.off('DiplomacyEventStarted', this.onDiplomacyEventStarted, this);
        engine.off('DiplomacyEventEnded', this.onDiplomacyEventEnded, this);
        engine.off('TurnBegin', this.onTurnBegin, this);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.cleanup();
        super.onDetach();
    }
    cleanup() {
        const manager = CityBannerManager.instance;
        manager.removeChildFromTracking(this);
        engine.off('BeforeUnload', this.onUnload, this);
        this.Root.removeEventListener('action-activate', this.onActivateEventListener);
        this.destroyWorldAnchor();
        this.componentID = ComponentID.getInvalidID();
    }
    onActivate() {
        if (this.componentID.owner == GameContext.localPlayerID) {
            UI.Player.selectCity(this.componentID);
        }
        else {
            const otherPlayer = Players.get(this.componentID.owner);
            if (!otherPlayer) {
                console.error("city-banners: Invalid player library for owner of clicked city.");
                return;
            }
            if (otherPlayer.isMajor || otherPlayer.isMinor || otherPlayer.isIndependent) {
                //Enter diplomacy if clicking on the city of another major player
                //  Needing to add edge case check for if players haven't met since early age and distanct lands from each other so will never meet.
                //  	but close enough they spot each other.
                if (!Game.Diplomacy.hasMet(GameContext.localPlayerID, this.componentID.owner)) {
                    return;
                }
                DiplomacyManager.raiseDiplomacyHub(this.componentID.owner);
            }
        }
    }
    setVisibility(state) {
        if (this.isHidden) {
            return;
        }
        switch (state) {
            case RevealedStates.HIDDEN:
                this.Root.classList.add("hidden");
                break;
            case RevealedStates.REVEALED:
                this.Root.classList.remove("hidden");
                break;
            case RevealedStates.VISIBLE:
                this.Root.classList.remove("hidden");
                break;
            default:
                console.warn("Unknown visibility reveal type passed to city banner. vis: ", state, "  cid: ", ComponentID.toLogString(this.componentID));
                break;
        }
    }
    getVisibility() {
        return GameplayMap.getRevealedState(GameContext.localObserverID, this.location.x, this.location.y);
    }
    makeWorldAnchor(location) {
        this._worldAnchorHandle = WorldAnchors.RegisterFixedWorldAnchor(location, BANNER_ANCHOR_OFFSET);
        if (this._worldAnchorHandle !== null && this._worldAnchorHandle >= 0) {
            this.Root.setAttribute('data-bind-style-transform2d', `{{FixedWorldAnchors.offsetTransforms[${this._worldAnchorHandle}].value}}`);
            this.Root.setAttribute('data-bind-style-opacity', `{{FixedWorldAnchors.visibleValues[${this._worldAnchorHandle}]}}`);
        }
        else {
            console.error(`Failed to create world anchor for location`, location);
        }
    }
    destroyWorldAnchor() {
        if (!this._worldAnchorHandle) {
            return;
        }
        this.Root.removeAttribute('data-bind-style-transform2d');
        this.Root.removeAttribute('data-bind-style-opacity');
        WorldAnchors.UnregisterFixedWorldAnchor(this._worldAnchorHandle);
        this._worldAnchorHandle = null;
    }
    /**
     * Realizes the entire banner.
     */
    buildBanner() {
        const city = this.city;
        const playerID = this.componentID.owner;
        const player = Players.get(playerID);
        if (!player) {
            console.error("Unable to (re)build banner due to not having a valid player: ", playerID);
            return;
        }
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        let icon = Icon.getLeaderPortraitIcon(player.leaderType);
        if (getCurrentSettings().SettlementBannersMainIcon === "civilizationIcon") {
            const civType = player.civilizationType;
            const civIcon = Icon.getCivSymbolCSSFromCivilizationType(civType);
            icon = civIcon.replace("url('", "").replace("')", "");
        }
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        let bannerType = BannerType.village;
        if (city) {
            bannerType = (player.isMinor ? BannerType.cityState : city.isTown ? BannerType.town : BannerType.city);
        }
        let bonusDefinition = undefined;
        let civSymbol = "";
        if (bannerType == BannerType.cityState || bannerType == BannerType.village) {
            let cityStateColor = "";
            let cityStateIcon = "";
            const bonusType = Game.CityStates.getBonusType(playerID);
            bonusDefinition = GameInfo.CityStateBonuses.find(t => t.$hash == bonusType);
            const player = Players.get(this.componentID.owner);
            if (player) {
                let yieldType = '';
                let indCivType = GameInfo.Civilizations.lookup(player.civilizationType)?.CivilizationType;
                let imagePath = '';
                GameInfo.Independents.forEach(indDef => {
                    if (player.civilizationAdjective == indDef.CityStateName) {
                        indCivType = indDef.CityStateType;
                    }
                });
                switch (indCivType) {
                    case "MILITARISTIC":
                        cityStateColor = "#AF1B1C";
                        cityStateIcon = "url('fs://game/bonustype_militaristic.png')";
                        icon = "bonustype_militaristic.png";
                        break;
                    case "SCIENTIFIC":
                        yieldType = 'YIELD_SCIENCE';
                        cityStateColor = "#4D7C96";
                        cityStateIcon = "url('fs://game/bonustype_scientific.png')";
                        icon = "bonustype_scientific.png";
                        break;
                    case "ECONOMIC":
                        yieldType = 'YIELD_GOLD';
                        cityStateColor = "#FFD553";
                        cityStateIcon = "url('fs://game/bonustype_economic.png')";
                        icon = "bonustype_economic.png";
                        break;
                    case "CULTURAL":
                        yieldType = 'YIELD_CULTURE';
                        cityStateColor = "#892BB3";
                        cityStateIcon = "url('fs://game/bonustype_cultural.png')";
                        icon = "bonustype_cultural.png";
                        break;
                    // ------------------ MODIFIED BY NASU ------------------ START ------------------
                    default:
                        cityStateColor = "#ffffff";
                        cityStateIcon = "url('fs://game/Action_Attack.png')";
                        icon = "Action_Attack.png";
                        break;
                    // ------------------ MODIFIED BY NASU ------------------ END ------------------
                }
                imagePath = yieldType != '' ? "url(" + UI.getIconURL(yieldType, indCivType == "MILITARISTIC" ? "PLAYER_RELATIONSHIP" : "YIELD") + ")" : "url('fs://game/Action_Attack.png')";
                civSymbol = imagePath;
            }
            if (!bonusDefinition && bonusType != -1) {
                console.error(`city-banners: couldn't find definition for city-state bonus type ${bonusType}`);
            }
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            // independents type now disabled
            //this.realizeCityStateType(cityStateColor, cityStateIcon);
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
        else {
            civSymbol = Icon.getCivSymbolCSSFromPlayer(this.componentID);
        }
        const leaderType = player.leaderType;
        let tooltip = "";

        const leader = GameInfo.Leaders.lookup(leaderType);
        let leaderName = Locale.compose((leader == null) ? "LOC_LEADER_NONE_NAME" : leader.Name);
        const civName = Locale.compose(GameplayMap.getOwnerName(this.location.x, this.location.y));
        // if this is an IP village or a town or city captured by an IP, use the IP leader name
        if (bannerType == BannerType.village || ((bannerType == BannerType.town || bannerType == BannerType.city) && player.isIndependent)) {
            leaderName = Locale.compose(player.name);
        }
        // Display icon of suzerain if this is a minor civ and update tooltip to show suzerain and bonus given
        if (player.isMinor && player.Influence?.hasSuzerain) {
            const suzerain = player.Influence.getSuzerain();
            const suzerainPlayer = Players.get(suzerain);
            icon = Icon.getLeaderPortraitIcon(suzerainPlayer.leaderType);
            if (suzerainPlayer) {
                // ------------------ MODIFIED BY NASU ------------------ START ------------------
                if (getCurrentSettings().SettlementBannersMainIcon === "civilizationIcon") {
                    const civType = suzerainPlayer.civilizationType;
                    let civIcon = Icon.getCivSymbolCSSFromCivilizationType(civType);
                    icon = civIcon.replace("url('", "").replace("')", "");
                }
                // ------------------ MODIFIED BY NASU ------------------ END ------------------
                const suzerainLeaderName = Locale.compose(suzerainPlayer.name);
                tooltip = `<div>${suzerainLeaderName}</div><div>${civName}</div>`;
            }
            if (bonusDefinition?.Name) {
                const bonusDefinitionName = Locale.compose(bonusDefinition.Name);
                tooltip += `<div>${bonusDefinitionName}</div>`;
            }
            this.affinityUpdate();
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            this.elements.outerContainer.style.fxsBorderImageTint = UI.Player.getPrimaryColorValueAsString(this.componentID.owner);
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
        else {
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            if (city) {
                tooltip = `<div>${leaderName}</div><div>${civName}</div>`;
            }
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
        let name = "";
        if (city) {
            name = city.name;
        }
        else {
            if (player == null) {
                name = Locale.compose("LOC_TERM_NONE");
                console.error(`city-banners: buildBanner(): couldn't get player for independent with PlayerId ${this.componentID.owner}`);
            }
            else {
                name = Locale.compose(player.civilizationFullName);
            }
        }
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        if (bannerType == BannerType.village) {
            this.affinityUpdate();
        }
        this.realizePlayerColors();
        // color switcher for independents
        let primaryColor = UI.Player.getPrimaryColorValueAsString(playerID);
        let secondaryColor = UI.Player.getSecondaryColorValueAsString(playerID);
        let civIconColor = secondaryColor;
        let civIconBgColor = primaryColor;
        const hasSuzerain = player.isMinor && player.Influence?.hasSuzerain;
        if (hasSuzerain) {
            primaryColor = "rgb(255, 255, 255)";
            secondaryColor = "rgb(0, 0, 0)";
            const suzerain = player.Influence.getSuzerain();
            civIconColor = UI.Player.getSecondaryColorValueAsString(suzerain);
            civIconBgColor = UI.Player.getPrimaryColorValueAsString(suzerain);
        }
        if (player.isIndependent) {
            primaryColor = "rgb(0, 0, 0)";
            secondaryColor = "rgb(255, 255, 255)";
            let indyTypeColor = secondaryColor;
            if (getCurrentSettings().IndependentsIconColor === "bonusType") {
                let indCivType = GameInfo.Civilizations.lookup(player.civilizationType)?.CivilizationType;
                GameInfo.Independents.forEach(indDef => {
                    if (player.civilizationAdjective == indDef.CityStateName) {
                        indCivType = indDef.CityStateType;
                    }
                });
                switch (indCivType) {
                    case "MILITARISTIC":
                        indyTypeColor = "#AF1B1C";
                        break;
                    case "SCIENTIFIC":
                        indyTypeColor = "#4D7C96";
                        break;
                    case "ECONOMIC":
                        indyTypeColor = "#FFD553";
                        break;
                    case "CULTURAL":
                        indyTypeColor = "#892BB3";
                        break;
                }
            }
            else {
                indyTypeColor = secondaryColor;
            }
            civIconColor = indyTypeColor;
            civIconBgColor = primaryColor;
        }
        this.Root.style.setProperty('--player-color-primary', primaryColor);
        this.Root.style.setProperty('--player-color-secondary', secondaryColor);

        // root

        if (player.isMinor && player.Influence?.hasSuzerain){
            this.Root.style.top = '-3.1rem';
        }
        else if (this.city && this.city.isTown) {
            this.Root.style.top = '-2.7rem';
        }
        else {
            this.Root.style.top = '-3.0rem';
            
        }
        this.elements.container.style.margin = '0rem';
        this.elements.container.style.padding = '0rem';
        this.elements.container.style.top = '0rem';

        
        // vanilla border, now disabled
        this.elements.cityStateRing.style.display = 'none';

        // useless extra shape, now disabled
        this.elements.outerContainer.style.display = 'none';

        // hostility border
        this.elements.cityStateBorder.style.display = 'none';
        this.elements.cityStateBorder.style.filter = `blur(12px)`;
        this.elements.cityStateBorder.style.animation = 'pulseGlow 1.5s ease-in-out infinite';
        this.elements.cityStateBorder.style.fxsBorderImageTint = secondaryColor;
        if (this.city && !this.city.isTown) {
            this.elements.cityStateBorder.style.top = '-1.00rem';
        }
        else{
            this.elements.cityStateBorder.style.top = '-0.25rem';
        }
       

        // banner pill shape
        this.elements.cityNameContainer.style.backgroundColor = primaryColor;
        this.elements.cityNameContainer.style.height = "2.5rem";
        this.elements.cityNameContainer.style.borderRadius = '1.5rem';
        this.elements.cityNameContainer.style.border = `0.125rem solid ${secondaryColor}`;
        this.elements.cityNameContainer.style.marginTop = '0rem';
        this.elements.cityNameContainer.style.paddingRight = '0.45rem';
        if (this.city && this.city.isTown) {
            this.elements.cityNameContainer.style.paddingRight = '0.575rem';
        }
        if (player.isMinor && player.Influence?.hasSuzerain){
            this.elements.cityNameContainer.style.paddingRight = '0.10rem';
        }
        
        // settlement name
        this.elements.cityName.style.padding = '0rem 0rem';
        this.elements.cityName.style.letterSpacing = '0.03rem';
        this.elements.cityName.style.marginLeft = '0rem';
        this.elements.cityName.style.marginRight = '0.5rem';
        this.elements.cityName.style.marginTop = '0rem';
        this.elements.cityName.style.fontWeight = '400';
        this.elements.cityName.style.fontSize = '0.8rem';
        this.elements.cityName.style.textShadow = 'none';
        this.elements.cityName.style.webkitTextStroke = '0.02rem ' + secondaryColor;
        this.elements.cityName.style.textStroke = '0.02rem ' + secondaryColor;
        
        // leader portrait frills, now disabled
        this.elements.portraitBgHex.style.display = 'none';
        this.elements.portraitBgFrame.style.display = 'none';

        // leader portrait
        this.elements.portrait.style.display = 'flex';

        this.elements.portrait.style.top = '0rem';
        this.elements.portrait.style.borderRadius = '50%';
        this.elements.portrait.style.alignSelf = 'center';
        this.elements.portrait.style.justifySelf = 'center';
        this.elements.portrait.style.marginLeft = '0rem';
        
        if (hasSuzerain) {
            this.elements.portrait.style.marginRight = '0.1rem';
        }
        else {
            this.elements.portrait.style.marginRight = '-0.2rem';
        }
        if (getCurrentSettings().SettlementBannersMainIcon === "civilizationIcon") {
            this.elements.portrait.style.backgroundColor = civIconBgColor;
            this.elements.portrait.style.width = '2.0rem';
            this.elements.portrait.style.height = '2.0rem';
            this.elements.portrait.style.padding = '0.25rem';
            if (hasSuzerain) {
                this.elements.portrait.style.width = '1.5rem';
                this.elements.portrait.style.height = '1.5rem';
                this.elements.portrait.style.marginLeft = '0.25rem';
            }
        }
        else {
            if (player.isIndependent) {
                this.elements.portrait.style.width = '2.0rem';
                this.elements.portrait.style.height = '2.0rem';
                this.elements.portrait.style.padding = '0.25rem';
            }
            else{
                this.elements.portrait.style.width = '2.5rem';
                this.elements.portrait.style.height = '2.5rem';
                this.elements.portrait.style.paddingBottom = '0.25rem';
            }
            
        }
        
        this.elements.portraitImg.style.position = 'static';
        this.elements.portraitImg.style.width = '100%';
        this.elements.portraitImg.style.height = '100%';
        this.elements.portraitImg.style.left = '0rem';
        this.elements.portraitImg.style.right = '0rem';
        this.elements.portraitImg.style.top = '0rem';
        this.elements.portraitImg.style.bottom = '0rem';
        this.elements.portraitImg.style.backgroundSize = 'cover';
        if (getCurrentSettings().SettlementBannersMainIcon === "civilizationIcon") {
            this.elements.portraitImg.style.fxsBackgroundImageTint = civIconColor;
        }

        // vslot
        this.elements.vSlot.style.height = '100%';
        this.elements.vSlot.style.marginLeft = '0.1rem';
        this.elements.vSlot.classList.remove('max-h-10');

        // first row
        this.elements.hSlot1.style.display = 'flex';
        this.elements.hSlot1.style.alignItems = 'center';
        this.elements.hSlot1.style.justifyContent = 'start';
        this.elements.hSlot1.style.left = '0rem';
        this.elements.hSlot1.style.right = '0rem';
        this.elements.hSlot1.style.top = '0rem';
        this.elements.hSlot1.style.bottom = '0rem';
        this.elements.hSlot1.style.height = '100%';
        this.elements.hSlot1.style.overflow = 'visible';
        this.elements.hSlot1.classList.remove('max-h-10');
        this.elements.hSlot1.style.gap = '0.5rem';
        this.elements.hSlot1.style.margin = '0rem';
        this.elements.hSlot1.style.padding = '0rem';

        // population
        this.elements.growthQueueContainer.style.marginBottom = '1.35rem';
        this.elements.growthQueueContainer.style.marginTop = '0rem';
        this.elements.growthQueueContainer.style.marginLeft = '1.0rem';
        this.elements.growthQueueContainer.style.alignItems = 'center';
        this.elements.growthQueueContainer.style.justifyContent = 'center';
        this.elements.growthQueueContainer.style.alignSelf = 'center';
        this.elements.growthQueueContainer.style.justifySelf = 'center';
        this.elements.growthQueueContainer.style.top = '0rem';

        this.elements.growthQueueMeter.style.marginTop = '0rem';
        this.elements.growthQueueMeter.style.marginBottom = '0rem';
        this.elements.growthQueueMeter.classList.remove('size-9');
        this.elements.growthQueueMeter.style.width = '2.75rem';
        this.elements.growthQueueMeter.style.height = '2.75rem';

        if (getCurrentSettings().SettlementPopulationElementMode === "growthIcon") {
            this.elements.popCount.style.display = 'none';
            const populationRing = this.elements.container.querySelector('.city-banner__population-ring');
            let populationImg = populationRing ? populationRing.querySelector('.city-banner__population-img') : null;
            if (populationRing && !populationImg) {
                populationImg = document.createElement('div');
                populationImg.className = 'city-banner__population-img size-4 self-center';
                populationRing.appendChild(populationImg);
            }
            if (populationRing && populationImg) {
                populationImg.style.backgroundSize = '87.5%';
                populationImg.style.backgroundRepeat = 'no-repeat';
                populationImg.style.backgroundPosition = 'center';
                populationImg.style.margin = '0';
                populationImg.style.width = '65%';
                populationImg.style.height = '65%';
                populationImg.style.position = 'relative';

                populationRing.style.display = 'flex';
                populationRing.style.justifyContent = 'center';
                populationRing.style.alignItems = 'center';
                populationRing.style.overflow = 'visible';
                populationRing.style.position = 'relative';
                populationRing.style.padding = '0';
            }
        }
        else {
            this.elements.popCount.style.backgroundSize = '0%';
            this.elements.popCount.style.fontSize = '0.75rem';
            this.elements.popCount.style.display = 'flex';
            this.elements.popCount.style.justifyContent = 'center';
            this.elements.popCount.style.alignItems = 'center';
            this.elements.popCount.style.width = '100%';
            this.elements.popCount.style.height = '100%';
            this.elements.popCount.style.letterSpacing = '-0.05rem';
        }

        switch (getCurrentSettings().TurnCountersMode) {
            case 'counters': {
                this.elements.growthQueueTurns.style.backgroundImage = 'none';
                this.elements.growthQueueTurns.style.backgroundColor = primaryColor;
                this.elements.growthQueueTurns.style.border = `0.1rem solid ${secondaryColor}`;
                this.elements.growthQueueTurns.style.borderRadius = '50%';
                this.elements.growthQueueTurns.style.width = '1.5rem';
                this.elements.growthQueueTurns.style.height = '1.5rem';
                this.elements.growthQueueTurns.style.margin = '0rem';
                this.elements.growthQueueTurns.style.padding = '0rem';
                this.elements.growthQueueTurns.style.display = 'flex';
                this.elements.growthQueueTurns.style.justifyContent = 'center';
                this.elements.growthQueueTurns.style.alignItems = 'center';
                this.elements.growthQueueTurns.style.fontSize = '0.6rem';
                this.elements.growthQueueTurns.style.position = 'absolute';
                this.elements.growthQueueTurns.style.top = `-4.3rem`;
                this.elements.growthQueueTurns.style.left = '-0.55rem';
                this.elements.growthQueueTurns.style.color = secondaryColor;

                const growthTurnParent = this.elements.growthQueueContainer.querySelector('.city-banner__turn');
                growthTurnParent.style.margin = '0rem';
                growthTurnParent.style.padding = '0rem';
                growthTurnParent.style.top = '0rem';
                growthTurnParent.style.position = 'relative';
                break;
            }
            case 'tooltips': {
                this.elements.growthQueueTurns.style.display = 'none';
                break;
            }
        }
        
        // production
        this.elements.productionQueueContainer.style.marginBottom = '1.35rem';
        this.elements.productionQueueContainer.style.marginTop = '0rem';
        this.elements.productionQueueContainer.style.marginLeft = '0.85rem';
        this.elements.productionQueueContainer.style.marginRight = '-0.1rem';
        this.elements.productionQueueContainer.style.alignItems = 'center';
        this.elements.productionQueueContainer.style.justifyContent = 'center';
        this.elements.productionQueueContainer.style.alignSelf = 'center';
        this.elements.productionQueueContainer.style.justifySelf = 'center';
        this.elements.productionQueueContainer.style.top = '0rem';

        this.elements.productionQueueMeter.style.marginTop = '0rem';
        this.elements.productionQueueMeter.style.marginBottom = '0rem';
        this.elements.productionQueueMeter.classList.remove('size-9');
        this.elements.productionQueueMeter.style.width = '2.75rem';
        this.elements.productionQueueMeter.style.height = '2.75rem';
        this.elements.productionQueueMeter.style.display = 'flex';
        this.elements.productionQueueMeter.style.alignItems = 'center';
        this.elements.productionQueueMeter.style.justifyContent = 'center';

        this.elements.prodIcon.style.margin = '0';
        this.elements.prodIcon.style.display = 'flex';
        this.elements.prodIcon.style.justifyContent = 'center';
        this.elements.prodIcon.style.alignItems = 'center';
        this.elements.prodIcon.style.height = '45%';
        this.elements.prodIcon.style.width = '45%';

        switch (getCurrentSettings().TurnCountersMode) {
            case 'counters': {
                this.elements.productionQueueTurns.style.backgroundImage = 'none';
                this.elements.productionQueueTurns.style.backgroundColor = primaryColor;
                this.elements.productionQueueTurns.style.border = `0.1rem solid ${secondaryColor}`;
                this.elements.productionQueueTurns.style.borderRadius = '50%';
                this.elements.productionQueueTurns.style.width = '1.5rem';
                this.elements.productionQueueTurns.style.height = '1.5rem';
                this.elements.productionQueueTurns.style.margin = '0rem';
                this.elements.productionQueueTurns.style.padding = '0rem';
                this.elements.productionQueueTurns.style.display = 'flex';
                this.elements.productionQueueTurns.style.justifyContent = 'center';
                this.elements.productionQueueTurns.style.alignItems = 'center';
                this.elements.productionQueueTurns.style.fontSize = '0.6rem';
                this.elements.productionQueueTurns.style.position = 'relative';
                this.elements.productionQueueTurns.style.top = '-4.3rem';
                this.elements.productionQueueTurns.style.left = '0.35rem';
                this.elements.productionQueueTurns.style.color = secondaryColor;

                const prodTurnParent = this.elements.productionQueueContainer.querySelector('.city-banner__turn');
                prodTurnParent.style.margin = '0rem';
                prodTurnParent.style.padding = '0rem';
                prodTurnParent.style.top = '0rem';
                prodTurnParent.style.position = 'relative';
                break;
            }
            case 'tooltips': {
                this.elements.productionQueueTurns.style.display = 'none';
                break;
            }
        }

        // Move conquered, unrest, and razing from their original location
        const targetSlot = this.elements.hSlot1;
        const statusSlot = this.elements.container.parentElement.querySelector('fxs-hslot.items-center');
        if (statusSlot) {
            targetSlot.appendChild(this.elements.conqueredIcon.parentElement);
            targetSlot.appendChild(this.elements.unrestIcon.parentElement);
            targetSlot.appendChild(this.elements.razingIcon.parentElement);
            statusSlot.remove();
        }
        
        // settlement type
        if (getCurrentSettings().SettlementTypeIcon === "showIcon") {
            let settlementTypeIcon = this.elements.hSlot1.querySelector('.city-banner__settlement-type');
            if (!settlementTypeIcon) {
                settlementTypeIcon = document.createElement('div');
                settlementTypeIcon.className = 'city-banner__settlement-type';
                this.elements.hSlot1.insertBefore(settlementTypeIcon, this.elements.hSlot1.firstChild);
            }
            if (this.city && !this.city.isCapital) {
                if (this.city.isTown) {
                    settlementTypeIcon.style.backgroundImage = 'url(fs://game/nasuellia-alt-unit-flags-and-bars/icons/town.png)';
                    settlementTypeIcon.style.pointerEvents = 'auto';
                    settlementTypeIcon.setAttribute('data-tooltip-content', 'LOC_CAPITAL_SELECT_PROMOTION_NONE');
                } else {
                    settlementTypeIcon.style.backgroundImage = 'url(fs://game/nasuellia-alt-unit-flags-and-bars/icons/city.png)';
                    settlementTypeIcon.style.pointerEvents = 'auto';
                    settlementTypeIcon.setAttribute('data-tooltip-content', 'LOC_CAPITAL_SELECT_PROMOTION_CITY');
                }
                settlementTypeIcon.style.width = '1.10rem';
                settlementTypeIcon.style.height = '1.10rem';
                settlementTypeIcon.style.backgroundSize = 'cover';
                settlementTypeIcon.style.marginRight = '0.33rem';
                settlementTypeIcon.style.display = 'flex';
                settlementTypeIcon.style.alignSelf = 'center';
                settlementTypeIcon.style.fxsBackgroundImageTint = secondaryColor;
            } else {
                settlementTypeIcon.style.display = 'none';
            }
        }

        // Reorder all elements in hSlot1 to the desired sequence
        targetSlot.appendChild(this.elements.urbanReligionSymbolBackground);  // Religion (urban)
        targetSlot.appendChild(this.elements.ruralReligionSymbolBackground);  // Religion (rural)
        targetSlot.appendChild(this.elements.statusContainer);                // Status
        targetSlot.appendChild(this.elements.unrestIcon.parentElement);       // Unrest
        targetSlot.appendChild(this.elements.conqueredIcon.parentElement);    // Conquered
        targetSlot.appendChild(this.elements.razingIcon.parentElement);       // Razing

        // add tooltips
        this.elements.conqueredIcon.style.pointerEvents = 'auto';
        this.elements.conqueredIcon.setAttribute('data-tooltip-content', 'LOC_CITY_BANNER_CONQUERED_TOOLTIP');
        this.elements.unrestIcon.style.pointerEvents = 'auto';
        this.elements.unrestIcon.setAttribute('data-tooltip-content', 'LOC_CITY_UNREST');
        this.elements.razingIcon.style.pointerEvents = 'auto';
        this.elements.razingIcon.setAttribute('data-tooltip-content', 'LOC_UI_CITY_DETAILS_CITY_BEING_RAZED');
        
        // capital symbol
        this.elements.capitalStar.style.width = '1.10rem';
        this.elements.capitalStar.style.height = '1.10rem';
        this.elements.capitalStar.style.marginRight = '0.2rem';
        this.elements.capitalStar.style.marginTop = '0rem';
        this.elements.capitalStar.style.backgroundImage = `url('fs://game/nasuellia-alt-unit-flags-and-bars/icons/star.png')`;
        this.elements.capitalStar.style.fxsBackgroundImageTint = secondaryColor;
        this.elements.capitalStar.style.pointerEvents = 'auto';
        this.elements.capitalStar.setAttribute('data-tooltip-content', 'LOC_CAPITAL_SELECT_PROMOTION_CAPITAL');

        // second row
        this.elements.hSlot2.style.height = '0%';
        this.elements.hSlot2.classList.remove('max-h-10');
        this.elements.hSlot2.style.top = '0rem';
        this.elements.hSlot2.style.left = '0rem';

        // happiness container
        this.elements.statusContainer.style.height = '1.15rem';
        this.elements.statusContainer.style.width = '1.15rem';
        this.elements.statusContainer.style.marginRight = '0.4rem';

        // happiness
        this.elements.statusIcon.style.backgroundSize = '100%';
        this.elements.statusIcon.style.position = 'static';
        this.elements.statusIcon.style.width = '100%';
        this.elements.statusIcon.style.height = '100%';
        this.elements.statusIcon.style.fxsBackgroundImageTint = secondaryColor;
        
        this.elements.statusBg.style.display = 'none';
        this.elements.statusBg.classList.remove('absolute');

        // religion
        this.elements.urbanReligionSymbolBackground.style.marginRight = '0rem';
        this.elements.urbanReligionSymbolBackground.style.border = 'none';
        this.elements.urbanReligionSymbolBackground.style.backgroundColor = 'transparent';
        this.elements.urbanReligionSymbolBackground.style.height = '1.15rem';
        this.elements.urbanReligionSymbolBackground.style.width = '1.15rem';

        this.elements.urbanReligionSymbol.style.position = 'static';
        this.elements.urbanReligionSymbol.style.width = '100%';
        this.elements.urbanReligionSymbol.style.height = '100%';
        this.elements.urbanReligionSymbol.style.marginTop = '0rem';
        this.elements.urbanReligionSymbol.style.marginLeft = '0rem';
        
        this.elements.ruralReligionSymbolBackground.style.marginRight = '0rem';
        this.elements.ruralReligionSymbolBackground.style.border = 'none';
        this.elements.ruralReligionSymbolBackground.style.backgroundColor = 'transparent';
        this.elements.ruralReligionSymbolBackground.style.height = '1.15rem';
        this.elements.ruralReligionSymbolBackground.style.width = '1.15rem';
        this.elements.ruralReligionSymbolBackground.style.marginRight = '0.4rem';

        this.elements.ruralReligionSymbol.style.position = 'static';
        this.elements.ruralReligionSymbol.style.width = '100%';
        this.elements.ruralReligionSymbol.style.height = '100%';
        this.elements.ruralReligionSymbol.style.marginTop = '0rem';
        this.elements.ruralReligionSymbol.style.marginLeft = '0rem';

        // force hide religion for independents
        if (player.isIndependent) {
            this.elements.urbanReligionSymbolBackground.style.display = 'none';
            this.elements.ruralReligionSymbolBackground.style.display = 'none';
            this.elements.cityNameContainer.style.paddingRight = '0.75rem';
        }

        // city state
        this.elements.cityStateColor.style.display = 'none';
        this.elements.cityStateIcon.style.display = 'none';

        // conquered
        this.elements.conqueredIcon.style.position = 'static';
        this.elements.conqueredIcon.style.height = '1.15rem';
        this.elements.conqueredIcon.style.width = '1.15rem';
        this.elements.conqueredIcon.style.transform = 'none';
        this.elements.conqueredIcon.style.marginTop = '0rem';
        this.elements.conqueredIcon.style.marginRight = '0.4rem';
        this.elements.conqueredIcon.style.position = 'static';
        this.elements.conqueredIcon.style.left = '0rem';
        this.elements.conqueredIcon.style.backgroundImage = `url('fs://game/nasuellia-alt-unit-flags-and-bars/icons/chain.png')`;
        this.elements.conqueredIcon.style.fxsBackgroundImageTint = secondaryColor;
        
        // unrest
        this.elements.unrestTimeContainer.style.display = 'none';

        this.elements.unrestIcon.style.position = 'static';
        this.elements.unrestIcon.style.height = '1.15rem';
        this.elements.unrestIcon.style.width = '1.15rem';
        this.elements.unrestIcon.style.marginTop = '0rem';
        this.elements.unrestIcon.style.marginLeft = '0rem';
        this.elements.unrestIcon.style.marginRight = '0.4rem';
        this.elements.unrestIcon.style.marginLeft = '-0.5rem';
        this.elements.unrestIcon.style.backgroundImage = `url('fs://game/nasuellia-alt-unit-flags-and-bars/icons/fist.png')`;
        this.elements.unrestIcon.style.fxsBackgroundImageTint = secondaryColor;

        // razing
        this.elements.razingTimeContainer.style.display = 'none';

        this.elements.razingIcon.style.position = 'static';
        this.elements.razingIcon.style.height = '1.15rem';
        this.elements.razingIcon.style.width = '1.15rem';
        this.elements.razingIcon.style.marginTop = '0rem';
        this.elements.razingIcon.style.marginLeft = '0rem';
        this.elements.razingIcon.style.backgroundImage = `url('fs://game/nasuellia-alt-unit-flags-and-bars/icons/fire.png')`;
        this.elements.razingIcon.style.fxsBackgroundImageTint = secondaryColor;
        
        this.originalBannerIcon = icon;
        this.diplomacyUpdate();
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        if (city) {
            this.setPopulation(city.population);
            this.realizeBuilds();
            this.realizeHappiness();
            this.realizeReligion();
        }
        const bannerData = {
            name: name,
            icon,
            tooltip,
            bannerType,
        };
        this.setCityInfo(bannerData);
        this.realizeCivHeraldry(civSymbol);
        this.updateConqueredIcon();
    }
    /**
     * Set the static information inside of the city banner.
     * @param {BannerData} data All string data is locale translated.
     */
    setCityInfo(data) {
        const name = data.name ?? "LOC_CITY_NAME_UNSET";
        const icon = data.icon ?? "fs://game/base-standard/ui/icons/leaders/leader_portrait_unknown.png";
        const { capitalIndicator, container, cityName, portrait, portraitIcon, } = this.elements;
        cityName.setAttribute("data-l10n-id", name);
        portraitIcon.style.backgroundImage = `url('${icon}')`;
        portrait.setAttribute("data-tooltip-content", data.tooltip);
        // remove all of the possible 4 variant classes
        this.Root.classList.remove('city-banner--town', 'city-banner--city', 'city-banner--city-other', 'city-banner--citystate');
        // and now add the appropriate variant class
        if (data.bannerType == BannerType.town) {
            this.Root.classList.add('city-banner--town');
        }
        else {
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            //container.setAttribute("data-tooltip-content", data.tooltip);
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
            if (data.bannerType == BannerType.cityState) {
                this.Root.classList.add("city-banner--citystate");
            }
            else if (data.bannerType == BannerType.village) {
                // village is based on town
                this.Root.classList.add("city-banner--village", "city-banner--town");
            }
            else {
                const isLocalPlayerCity = this.componentID.owner === GameContext.localObserverID;
                this.Root.classList.toggle('city-banner--city', isLocalPlayerCity);
                this.Root.classList.toggle('city-banner--city-other', !isLocalPlayerCity);
                if (this.city) {
                    capitalIndicator.classList.toggle('hidden', !this.city.isCapital);
                    // don't show the capital indicator for a city captured by an independent power
                    const player = Players.get(this.componentID.owner);
                    if (player && player.isIndependent) {
                        capitalIndicator.classList.add('hidden');
                    }
                }
            }
        }
    }
    setPopulation(population) {
        this.elements.popCount.textContent = population.toString();
    }
    setProduction(data) {
        const { productionQueue, productionQueueIcon, productionQueueTurns, productionQueueMeter } = this.elements;
        if (data && data.turnsLeft > 0) {
            productionQueue.classList.remove('queue-none');
            productionQueueIcon.style.backgroundImage = `url('${Icon.getProductionIconFromHash(data.hash)}')`;
            productionQueueIcon.classList.toggle('city-banner__queue-img--unit', (data.kind == ProductionKind.UNIT));
            const name = (data.kind == ProductionKind.UNIT) ? GameInfo.Units.lookup(data.hash)?.Name : GameInfo.Constructibles.lookup(data.hash)?.Name;
            if (!name) {
                console.error(`City Banner Production Icon Tooltip: No name could be found for data with hash ${data.hash}`);
            }
            else {
                // ------------------ MODIFIED BY NASU ------------------ START ------------------
                productionQueue.setAttribute("data-tooltip-content", `<div>${Locale.compose("LOC_UI_CITY_BANNER_PRODUCTION")}</div><div>${Locale.compose(name)}</div><div>${data.turnsLeft.toString()} ${Locale.compose("LOC_UI_CITY_INSPECTOR_TURNS")}</div>`);
                // ------------------ MODIFIED BY NASU ------------------ END ------------------
            }
        }
        else {
            productionQueue.classList.add('queue-none');
        }
        // INVESTIGATE - Should these use the infinity (∞) icon/symbol instead? 
        productionQueueTurns.textContent = (data) ? data.turnsLeft.toString() : "0";
        productionQueueMeter.setAttribute("value", (data) ? data.percentLeft.toString() : "0");
    }
    setFood(turnsLeft, current, nextTarget) {
        const { growthQueueContainer, growthQueueMeter, growthQueueTurns } = this.elements;
        if (turnsLeft >= 0) {
            growthQueueMeter.setAttribute("value", current.toString());
            growthQueueMeter.setAttribute("max-value", nextTarget.toString());
            growthQueueTurns.innerHTML = turnsLeft.toString();
            growthQueueTurns.classList.remove('hidden');
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            this.elements.growthQueueContainer.setAttribute("data-tooltip-content", `<div>${Locale.compose("LOC_UI_CITY_DETAILS_GROWTH_TAB")}:</div><div>${turnsLeft.toString()} ${Locale.compose("LOC_UI_CITY_INSPECTOR_TURNS")}</div><div>-------------------------</div><div>${Locale.compose("LOC_CAPITAL_SELECT_POPULATION")}: ${this.city.population}</div><div>${Locale.compose("LOC_ATTR_YIELDS_FROM_URBAN_POPULATION")}: ${this.city.urbanPopulation}</div><div>${Locale.compose("LOC_ATTR_YIELDS_FROM_RURAL_POPULATION")}: ${this.city.ruralPopulation}</div>`);
            if (getCurrentSettings().SettlementPopulationElementMode === "growthIcon") {
                const populationImg = this.elements.container.querySelector('.city-banner__population-ring .city-banner__population-img');
                populationImg.style.backgroundImage = 'url(fs://game/nasuellia-alt-unit-flags-and-bars/icons/growing.png)';
            }
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
        else {
            growthQueueTurns.classList.add('hidden');
            growthQueueMeter.setAttribute("value", "0");
            growthQueueMeter.setAttribute("max-value", "0");
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            growthQueueTurns.innerHTML = "-";
            growthQueueTurns.style.display = 'none';
            this.elements.growthQueueContainer.setAttribute("data-tooltip-content", `<div>${Locale.compose("LOC_UI_CITY_STATUS_STAGNANT_GROWTH")}`);
            if (getCurrentSettings().SettlementPopulationElementMode === "growthIcon") {
                const populationImg = this.elements.container.querySelector('.city-banner__population-ring .city-banner__population-img');
                populationImg.style.backgroundImage = 'url(fs://game/nasuellia-alt-unit-flags-and-bars/icons/not_growing.png)';
            }
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        const isLocalPlayerCity = this.componentID.owner === GameContext.localObserverID;
        if (!isLocalPlayerCity) {
            this.elements.growthQueueTurns.style.display = 'none';
        }
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
    }
    realizePopulation() {
        if (this.city) {
            this.setPopulation(this.city.population);
        }
    }
    realizeCityStateType(color, icon) {
        const iconDiv = MustGetElement(".city-banner__city-state-icon", this.Root);
        iconDiv.style.backgroundImage = icon;
        iconDiv.style.fxsBackgroundImageTint = color;
    }
    realizePlayerColors() {
        let playerColorPri = UI.Player.getPrimaryColorValueAsString(this.componentID.owner);
        let playerColorSec = UI.Player.getSecondaryColorValueAsString(this.componentID.owner);
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        //Prevent unreadable color combos
        /*
        if (playerColorPri == playerColorSec) {
            playerColorPri = "rgb(155, 0, 0)";
        }
         */
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.Root.style.setProperty('--player-color-primary', playerColorPri);
        this.Root.style.setProperty('--player-color-secondary', playerColorSec);
        this.Root.style.display = "flex";
    }
    realizeCivHeraldry(icon) {
        this.civPatternElements ?? (this.civPatternElements = this.Root.getElementsByClassName('city-banner__pattern'));
        const civPattern = Icon.getCivLineCSSFromPlayer(this.componentID);
        for (let i = 0; i < this.civPatternElements.length; i++) {
            this.civPatternElements[i].style.backgroundImage = civPattern;
        }
        this.civSymbols ?? (this.civSymbols = this.Root.getElementsByClassName('city-banner__symbol'));
        for (let i = 0; i < this.civSymbols.length; i++) {
            this.civSymbols[i].style.backgroundImage = icon;
        }
    }
    realizeReligion() {
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.elements.urbanReligionSymbolBackground.style.display = 'none';
        this.elements.ruralReligionSymbolBackground.style.display = 'none';
        this.stopUrbanFlipTwitch(this.elements.urbanReligionSymbolBackground);
        this.stopRuralFlipTwitch(this.elements.ruralReligionSymbolBackground);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        if (this.city) {
            // disable the religion section of the banner for starters
            this.Root.classList.remove('city-banner--has-religion');
            // check for a majority religion in this city
            const religion = GameInfo.Religions.find(t => t.$hash == this.city?.Religion?.majorityReligion);
            if (religion) {
                // there's a majority religion, so use that icon
                const icon = UI.getIconCSS(religion.ReligionType, "RELIGION");
                this.elements.urbanReligionSymbol.style.backgroundImage = icon;
                this.elements.ruralReligionSymbol.style.backgroundImage = icon;
                // ------------------ MODIFIED BY NASU ------------------ START ------------------
                this.elements.urbanReligionSymbolBackground.style.display = 'flex';
                this.elements.ruralReligionSymbolBackground.style.display = 'flex';
                this.elements.urbanReligionSymbolBackground.style.pointerEvents = 'auto';
                this.elements.urbanReligionSymbolBackground.setAttribute('data-tooltip-content', `${religion.Name}`);
                this.elements.ruralReligionSymbolBackground.style.pointerEvents = 'auto';
                this.elements.ruralReligionSymbolBackground.setAttribute('data-tooltip-content', `${religion.Name}`);
                const cityReligionName = religion.ReligionType;
                const playerID = this.componentID.owner;
                const player = Players.get(playerID);
                const isLocalPlayerCity = this.componentID.owner === GameContext.localObserverID;
                if (player && isLocalPlayerCity) {
                    const playerReligion = player.Religion;
                    if (playerReligion) {
                        const playerReligionDef = GameInfo.Religions.lookup(playerReligion.getReligionType() ?? null);
                        if (playerReligionDef) {
                            const playerReligionName = playerReligionDef.ReligionType;
                            if (playerReligionName !== cityReligionName) {
                                this.startUrbanFlipTwitch(this.elements.urbanReligionSymbolBackground);
                                this.startRuralFlipTwitch(this.elements.ruralReligionSymbolBackground)
                            }
                        }
                    }
                }
                // ------------------ MODIFIED BY NASU ------------------ END ------------------
                this.elements.cityName.classList.add("city-banner__icons-below-name");
                this.Root.classList.add('city-banner--has-religion');
            }
            else {
                // no majority, check if urban or rural religions exist
                const urbanReligion = GameInfo.Religions.find(t => t.$hash == this.city?.Religion?.urbanReligion);
                if (urbanReligion) {
                    const icon = UI.getIconCSS(urbanReligion.ReligionType, "RELIGION");
                    this.elements.urbanReligionSymbol.style.backgroundImage = icon;
                    // ------------------ MODIFIED BY NASU ------------------ START ------------------
                    this.elements.urbanReligionSymbolBackground.style.display = 'flex';
                    this.elements.urbanReligionSymbolBackground.style.pointerEvents = 'auto';
                    this.elements.urbanReligionSymbolBackground.setAttribute('data-tooltip-content', `${urbanReligion.Name}`);
                    const cityReligionName = urbanReligion.ReligionType;
                    const playerID = this.componentID.owner;
                    const player = Players.get(playerID);
                    const isLocalPlayerCity = this.componentID.owner === GameContext.localObserverID;
                    if (player && isLocalPlayerCity) {
                        const playerReligion = player.Religion;
                        if (playerReligion) {
                            const playerReligionDef = GameInfo.Religions.lookup(playerReligion.getReligionType() ?? null);
                            if (playerReligionDef) {
                                const playerReligionName = playerReligionDef.ReligionType;
                                if (playerReligionName !== cityReligionName) {
                                    this.startUrbanFlipTwitch(this.elements.urbanReligionSymbolBackground);
                                }
                            }
                        }
                    }
                    // ------------------ MODIFIED BY NASU ------------------ END ------------------
                    this.elements.cityName.classList.add("city-banner__icons-below-name");
                    this.Root.classList.add('city-banner--has-religion');
                }
                const ruralReligion = GameInfo.Religions.find(t => t.$hash == this.city?.Religion?.ruralReligion);
                if (ruralReligion) {
                    const icon = UI.getIconCSS(ruralReligion.ReligionType, "RELIGION");
                    this.elements.ruralReligionSymbol.style.backgroundImage = icon;
                    // ------------------ MODIFIED BY NASU ------------------ START ------------------
                    this.elements.ruralReligionSymbolBackground.style.display = 'flex';
                    //this.elements.ruralReligionSymbolBackground.style.filter = "fxs-color-tint(red)";
                    this.elements.ruralReligionSymbolBackground.setAttribute('data-tooltip-content', `${ruralReligion.Name}`);
                    const cityReligionName = ruralReligion.ReligionType;
                    const playerID = this.componentID.owner;
                    const player = Players.get(playerID);
                    const isLocalPlayerCity = this.componentID.owner === GameContext.localObserverID;
                    if (player && isLocalPlayerCity) {
                        const playerReligion = player.Religion;
                        if (playerReligion) {
                            const playerReligionDef = GameInfo.Religions.lookup(playerReligion.getReligionType() ?? null);
                            if (playerReligionDef) {
                                const playerReligionName = playerReligionDef.ReligionType;
                                if (playerReligionName !== cityReligionName) {
                                    this.startRuralFlipTwitch(this.elements.ruralReligionSymbolBackground)
                                }
                            }
                        }
                    }
                    // ------------------ MODIFIED BY NASU ------------------ END ------------------
                    this.elements.cityName.classList.add("city-banner__icons-below-name");
                    this.Root.classList.add('city-banner--has-religion');
                }
            }
        }
        else {
            console.error("City Banner missing city object when religion changed. cid: ", ComponentID.toLogString(this.componentID));
        }
    }
    realizeBuilds() {
        if (this.city) {
            const buildQueue = this.city.BuildQueue;
            const cityGrowth = this.city.Growth;
            if (buildQueue) {
                if (buildQueue.isEmpty) {
                    this.setProduction(null);
                }
                else {
                    let currentProductionTypeHash = buildQueue.currentProductionTypeHash;
                    if (currentProductionTypeHash == null) {
                        console.warn(`city.BuildQueue.CurrentProductionTypeHash returned null instead of hash.`);
                        currentProductionTypeHash = -1;
                    }
                    this.setProduction({
                        hash: buildQueue.currentProductionTypeHash,
                        turnsLeft: buildQueue.currentTurnsLeft,
                        percentLeft: buildQueue.getPercentComplete(currentProductionTypeHash),
                        kind: buildQueue.currentProductionKind
                    });
                }
            }
            else {
                console.error("City-banners: RealizeBuilds: city.BuildQueue was undefined");
            }
            if (cityGrowth) {
                const isExpanding = !this.city.isTown || cityGrowth.growthType == GrowthTypes.EXPAND;
                const turnsUntilGrowth = isExpanding ? cityGrowth.turnsUntilGrowth : -1;
                this.setFood(turnsUntilGrowth, cityGrowth.currentFood, cityGrowth.getNextGrowthFoodThreshold().value);
            }
            else {
                console.error("City-banners: RealizeBuilds: city.Growth was undefined");
            }
        }
        else {
            console.error("City Banner missing city object when production changed. cid: ", ComponentID.toLogString(this.componentID));
        }
    }
    realizeHappiness() {
        if (this.city) {
            const happiness = this.city.Yields?.getYield(YieldTypes.YIELD_HAPPINESS);
            if (happiness == undefined) {
                console.error("city-banners.ts: realizeHappiness() failed to find happiness yield for city cid: ", ComponentID.toLogString(this.componentID));
                return;
            }
            let happinessStatus = CityStatusType.happy;
            if (happiness < 0) {
                happinessStatus = CityStatusType.unhappy;
            }
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            if (happiness < -10) {
                happinessStatus = CityStatusType.angry;
            }
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
            if (this.city.isInfected) {
                happinessStatus = CityStatusType.plague;
            }
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            //const icon = UI.getIconURL(happinessStatus, "YIELD");
            //this.elements.statusIcon.style.backgroundImage = `url('${icon}')`;
            //this.elements.statusContainer.style.display = (happinessStatus === CityStatusType.happy && !this.city.isInfected) ? 'none' : 'flex';
            //const isLocalPlayerCity = this.componentID.owner === GameContext.localObserverID;
            //this.elements.cityName.classList.toggle("city-banner__status--hidden", !isLocalPlayerCity);
            if (happinessStatus === CityStatusType.happy && !this.city.isInfected) {
                this.elements.statusContainer.style.display = 'none';
            }
            else {
                this.elements.statusContainer.style.display = 'flex';
                if (happinessStatus === CityStatusType.unhappy){
                    this.elements.statusIcon.style.backgroundImage = `url('fs://game/nasuellia-alt-unit-flags-and-bars/icons/sadface.png')`;
                    this.elements.statusContainer.style.pointerEvents = 'auto';
                    this.elements.statusContainer.setAttribute('data-tooltip-content', 'LOC_UI_CITY_DETAILS_UNHAPPY');
                }
                else if (happinessStatus === CityStatusType.angry){
                    this.elements.statusIcon.style.backgroundImage = `url('fs://game/nasuellia-alt-unit-flags-and-bars/icons/angryface.png')`;
                    this.elements.statusContainer.style.pointerEvents = 'auto';
                    this.elements.statusContainer.setAttribute('data-tooltip-content', 'LOC_UI_CITY_DETAILS_ANGRY');
                }
                else if (happinessStatus === CityStatusType.plague){
                    this.elements.statusIcon.style.backgroundImage = `url('fs://game/nasuellia-alt-unit-flags-and-bars/icons/plagued.png')`;
                    this.elements.statusContainer.style.pointerEvents = 'auto';
                    this.elements.statusContainer.setAttribute('data-tooltip-content', 'LOC_TUTORIAL_PLAGUES_NAME');
                }
            }
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
            if (!this.city.Happiness) {
                console.error(`city-banners: City happiness is not valid, cid: ${ComponentID.toLogString(this.componentID)}`);
            }
            this.Root.classList.toggle("city-banner--unrest", this.city.Happiness?.hasUnrest);
            // Unrest turns are the number of turns a city has been in unrest, not the amount remaining so we need to calc it. Copied from town-unrest-display.ts
            const unrestTurns = this.city.Happiness?.turnsOfUnrest;
            if (unrestTurns != undefined && unrestTurns >= 0) {
                const remainingUnrest = Math.max(0, unrestTurns);
                this.elements.unrestTurns.innerHTML = remainingUnrest.toString();
            }
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            this.Root.classList.toggle('city-banner--razing', this.city.isBeingRazed);
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
            const razedTurns = this.city.getTurnsUntilRazed.toString();
            this.elements.razedTurns.innerHTML = razedTurns;
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            if (this.city.isBeingRazed) {
                this.elements.razingIcon.style.display = 'flex';
            }
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
        else {
            console.error("city-banners.ts: realizeHappiness() failed to have a valid city cid: ", ComponentID.toLogString(this.componentID));
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            this.elements.statusContainer.style.display = 'none';
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
    }

    // ------------------ MODIFIED BY NASU ------------------ START ------------------
    onDiplomacyEventStarted(data) {
        const actionType = data.actionType;
        if (actionType === -434781988) {
            setTimeout(() => {
                this.diplomacyUpdate();
            }, 250);
        }
    }

    onTurnBegin(data) {
        setTimeout(() => {
            this.diplomacyUpdate();
        }, 100);
    }

    diplomacyUpdate(){
        const player = Players.get(this.componentID.owner);
        if (player) {
            const atWar = player.Diplomacy.isAtWarWith(GameContext.localPlayerID);
            switch (getCurrentSettings().HostilityIndicator) {
                case 'iconSwap': {
                    if (atWar) {
                        this.startHostileIconSwap();
                    }
                    else {
                        this.stopHostileIconSwap();
                    }
                    break;
                }
                case 'flagGlow': {
                    if (atWar) {
                        this.elements.cityStateBorder.style.display = 'flex';
                    }
                    else {
                        this.elements.cityStateBorder.style.display = 'none'
                    }
                    break;
                }
                case 'disabled': {
                    break;
                }
            }
        }
    }

    startUrbanFlipTwitch(view) {
        if (this.urbanReligionAnimationTracker && this.urbanReligionAnimationLoopTimer !== null) {
            clearTimeout(this.urbanReligionAnimationLoopTimer);
            view.style.transform = 'rotate(0deg)';
        };

        const doFlip = () => {
            if (!this.urbanReligionAnimationTracker) {
                view.style.transform = 'scaleX(1)';
                return;
            }

            view.style.transition = 'transform 0.33s ease-out';
            view.style.transform = 'scaleX(-1)';

            setTimeout(() => {
                view.style.transform = 'scaleX(1)';
            }, 333);

            this.urbanReligionAnimationLoopTimer = setTimeout(doFlip, 3000);
        };

        this.urbanReligionAnimationTracker = true;
        doFlip();
    }

    stopUrbanFlipTwitch(view) {
        clearTimeout(this.urbanReligionAnimationLoopTimer);
        this.urbanReligionAnimationLoopTimer = null;
        this.urbanReligionAnimationTracker = false;
        view.style.transition = '';
        view.style.transform = 'scaleX(1)';
    }

    startRuralFlipTwitch(view) {
        if (this.ruralReligionAnimationTracker && this.ruralReligionAnimationLoopTimer !== null) {
            clearTimeout(this.ruralReligionAnimationLoopTimer);
            view.style.transform = 'rotate(0deg)';
        };

        const doFlip = () => {
            if (!this.ruralReligionAnimationTracker) {
                view.style.transform = 'scaleX(1)';
                return;
            }

            view.style.transition = 'transform 0.33s ease-out';
            view.style.transform = 'scaleX(-1)';

            setTimeout(() => {
                view.style.transform = 'scaleX(1)';
            }, 333);

            this.ruralReligionAnimationLoopTimer = setTimeout(doFlip, 3000);
        };

        this.ruralReligionAnimationTracker = true;
        doFlip();
    }

    stopRuralFlipTwitch(view) {
        clearTimeout(this.ruralReligionAnimationLoopTimer);
        this.ruralReligionAnimationLoopTimer = null;
        this.ruralReligionAnimationTracker = false;
        view.style.transition = '';
        view.style.transform = 'scaleX(1)';
    }
    
    startHostileIconSwap() {
        if (!this.elements.portraitIcon || !this.originalBannerIcon) return;
        if (this.iconSwapSubscription) return;

        this.elements.portraitIcon.style.transition = 'transform 0.1s ease'; // Default for initial shrink

        var updateIcon = function() {
            var phase = animationClock.getPhase();
            if (phase === 'grow') {
                this.elements.portraitIcon.style.transform = 'scale(0.8)'; // Shrink before skull
                setTimeout(function() {
                    this.elements.portraitIcon.style.backgroundImage = 'url(fs://game/nasuellia-alt-unit-flags-and-bars/icons/war.png)';
                    this.elements.portraitIcon.style.transition = 'transform 0.1s ease-out'; // Overshoot
                    this.elements.portraitIcon.style.transform = 'scale(1.2)'; // Overshoot past 1.0
                }.bind(this), 100); // Half of grow’s 200ms phase
            } else if (phase === 'skull') {
                this.elements.portraitIcon.style.transition = 'transform 0.1s ease-in'; // Settle to 1.0
                this.elements.portraitIcon.style.transform = 'scale(1.0)';
                this.elements.portraitIcon.style.backgroundImage = 'url(fs://game/nasuellia-alt-unit-flags-and-bars/icons/war.png)';
            } else if (phase === 'shrink') {
                this.elements.portraitIcon.style.transform = 'scale(0.8)'; // Shrink before normal
                setTimeout(function() {
                    this.elements.portraitIcon.style.backgroundImage = 'url(' + this.originalBannerIcon + ')';
                    this.elements.portraitIcon.style.transition = 'transform 0.1s ease-out'; // Overshoot
                    this.elements.portraitIcon.style.transform = 'scale(1.2)'; // Overshoot past 1.0
                }.bind(this), 100); // Half of shrink’s 200ms phase
            } else if (phase === 'normal') {
                this.elements.portraitIcon.style.transition = 'transform 0.1s ease-in'; // Settle to 1.0
                this.elements.portraitIcon.style.transform = 'scale(1.0)';
                this.elements.portraitIcon.style.backgroundImage = 'url(' + this.originalBannerIcon + ')';
            }
        }.bind(this);

        this.iconSwapSubscription = updateIcon;
        animationClock.subscribe(updateIcon);
        animationClock.start();
    }

    stopHostileIconSwap() {
        if (this.iconSwapSubscription) {
            animationClock.unsubscribe(this.iconSwapSubscription);
            this.iconSwapSubscription = null;
            this.elements.portraitIcon.style.transition = '';
            this.elements.portraitIcon.style.transform = 'scale(1.0)';
            this.elements.portraitIcon.style.backgroundImage = 'url(' + this.originalBannerIcon + ')';
            console.warn('NASU: Stopped hostile swap, reset to normal');
        }
    }

    // ------------------ MODIFIED BY NASU ------------------ END ------------------

    affinityUpdate() {
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        const localPlayerID = GameContext.localPlayerID;
        const bannerOwner = Players.get(this.componentID.owner);
        if (bannerOwner) {
            const relationship = Game.IndependentPowers.getIndependentRelationship(this.componentID.owner, localPlayerID);
            if (relationship === IndependentRelationship.NOT_APPLICABLE) {
                console.warn("Village Banner unable to determine affinity relationship.");
                return;
            }
            const classList = this.Root.classList;
            if (relationship === IndependentRelationship.NEUTRAL) {
                this.elements.cityStateBorder.style.display = 'none'
                classList.toggle("city-banner--neutral", true);
            }
            else if (relationship === IndependentRelationship.FRIENDLY) {
                this.elements.cityStateBorder.style.display = 'none'
                classList.toggle("city-banner--friendly", true);
            }
            else if (relationship === IndependentRelationship.HOSTILE) {
                classList.toggle("city-banner--hostile", true);
            }
        }
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
    }
    capitalUpdate() {
        const capitalIndicator = MustGetElement('.city-banner__capital-star', this.Root);
        if (this.city) {
            capitalIndicator.classList.toggle('hidden', !this.city.isCapital);
            // don't show the capital indicator for a city captured by an independent power
            const player = Players.get(this.componentID.owner);
            if (player && player.isIndependent) {
                capitalIndicator.classList.add('hidden');
            }
        }
    }
    updateConqueredIcon() {
        if (this.city && (this.city.originalOwner != this.city.owner && this.city.owner == GameContext.localObserverID)) {
            const conqueredIcon = this.Root.querySelector(".city-banner__conquered-icon");
            if (!conqueredIcon) {
                console.error("city-banners: Unable to find element with class .city-banner__conquered-icon!");
                return;
            }
            conqueredIcon.setAttribute("data-tooltip-content", Locale.compose("LOC_CITY_BANNER_CONQUERED_TOOLTIP"));
            this.Root.classList.add("city-banner--conquered");
        }
        else {
            this.Root.classList.remove("city-banner--conquered");
        }
    }
    hide() {
        if (this.isHidden) {
            return;
        }
        this.isHidden = true;
        this.Root.classList.add("hidden");
    }
    show() {
        if (!this.isHidden) {
            return;
        }
        this.isHidden = false;
        this.setVisibility(this.getVisibility());
    }
    disable() {
        this.Root.classList.add("disabled");
        const elements = this.Root.querySelectorAll(this.inputSelector);
        if (elements.length == 0) {
            console.warn(`city-banners: disable(): Unable to disable city banner pieces. cid: ${ComponentID.toLogString(this.componentID)}`);
            return;
        }
        for (let i = 0; i < elements.length; i++) {
            elements[i].classList.add("disabled");
        }
    }
    enable() {
        this.Root.classList.remove("disabled");
        const elements = this.Root.querySelectorAll(this.inputSelector);
        if (elements.length == 0) {
            console.warn(`city-banners: disable(): Unable to disable city banner pieces. cid: ${ComponentID.toLogString(this.componentID)}`);
            return;
        }
        for (let i = 0; i < elements.length; i++) {
            elements[i].classList.remove("disabled");
        }
    }
    remove() {
        this.Destroy();
    }
}
Controls.define('city-banner', {
    createInstance: CityBannerComponent,
    description: 'City Banner',
    classNames: ['city-banner', 'allowCameraMovement'],
    styles: ['fs://game/base-standard/ui/city-banners/city-banners.css'],
    content: ['fs://game/base-standard/ui/city-banners/city-banners.html']
});
//# sourceMappingURL=file:///base-standard/ui/city-banners/city-banners.js.map
