/**
 * @file district-health.ts
 * @copyright 2023, Firaxis Games
 * @description A health bar that is attached to the 3D world and floats up.
 */
import DistrictHealthManager from '/base-standard/ui/district/district-health-manager.js';
import { Icon } from '/core/ui/utilities/utilities-image.js';
import { PlotCoord } from '/core/ui/utilities/utilities-plotcoord.js';
import { ComponentID } from '/core/ui/utilities/utilities-component-id.js';
import { MustGetElement } from '/core/ui/utilities/utilities-dom.js';
// ------------------ MODIFIED BY NASU ------------------ START ------------------
import { InterfaceMode } from '/core/ui/interface-modes/interface-modes.js';
// ------------------ MODIFIED BY NASU ------------------ END ------------------

// ------------------ MODIFIED BY NASU ------------------ START ------------------

class DebugUtils {
    static inspectObject(obj, label = "Object") {
        console.warn(`NASUDEBUG: ! ------------------ inspectObject BEGINNING ------------------ !`);
        if (!obj) {
            console.warn(`NASUDEBUG: ${label} is null or undefined`);
            return;
        }

        console.warn(`NASUDEBUG: ${label} exists, type:`, typeof obj);

        const ownProps = Object.getOwnPropertyNames(obj);
        console.warn(`NASUDEBUG: ${label} own properties count:`, ownProps.length);
        if (ownProps.length > 0) {
            ownProps.forEach(prop => {
                const value = obj[prop];
                const valueType = typeof value;
                const paramCount = valueType === 'function' ? `(${value.length} params)` : '';
                console.warn(`NASUDEBUG: ${label} own property: ${prop} (${valueType}${paramCount})`, value);
            });
        } else {
            console.warn(`NASUDEBUG: No own properties found in ${label}`);
        }

        const proto = Object.getPrototypeOf(obj);
        if (proto) {
            const protoProps = Object.getOwnPropertyNames(proto);
            console.warn(`NASUDEBUG: ${label} prototype properties count:`, protoProps.length);
            if (protoProps.length > 0) {
                protoProps.forEach(prop => {
                    const value = proto[prop];
                    const valueType = typeof value;
                    const paramCount = valueType === 'function' ? `(${value.length} params)` : '';
                    console.warn(`NASUDEBUG: ${label} proto property: ${prop} (${valueType}${paramCount})`, value);
                });
            } else {
                console.warn(`NASUDEBUG: No prototype properties found in ${label}`);
            }
        } else {
            console.warn(`NASUDEBUG: ${label} has no prototype`);
        }
        console.warn(`NASUDEBUG: ! ------------------ inspectObject END ------------------ !`);
    }
}

// ------------------ MODIFIED BY NASU ------------------ END ------------------
export class DistrictHealthBar extends Component {
    constructor() {
        super(...arguments);
        this._componentID = ComponentID.getInvalidID();
        this._worldAnchorHandle = null;
        this.progressBar = null;
        this.progressInk = null;
        this.civHexInner = null;
        this.civHexOuter = null;
        this.isCityCenter = false;
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.hslot = null;
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.MEDIUM_HEALTH_THRESHHOLD = .75; // thresholds for health bar color changes
        this.LOW_HEALTH_THRESHHOLD = .5;
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.interfaceModeChangedListener = () => { this.onInterfaceModeChanged(); };
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
    }

    // ------------------ MODIFIED BY NASU ------------------ START ------------------
    onInterfaceModeChanged() {
        const currentInterfaceMode = InterfaceMode.getCurrent();
        console.warn("NASU: onInterfaceModeChanged >> ", currentInterfaceMode);
        const diplomacyInterfaceModes = [
            "INTERFACEMODE_DIPLOMACY_HUB",
            "INTERFACEMODE_DIPLOMACY_PROJECT_REACTION",
            "INTERFACEMODE_DIPLOMACY_DIALOG",
            "INTERFACEMODE_PEACE_DEAL"
        ];
        
        if (diplomacyInterfaceModes.includes(currentInterfaceMode)) {
           this.hslot.style.visibility = 'hidden';
        }
        else {
            this.hslot.style.visibility = 'visible';
        }
    }
    // ------------------ MODIFIED BY NASU ------------------ END ------------------
    onAttach() {
        super.onAttach();
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        window.addEventListener('interface-mode-changed', this.interfaceModeChangedListener);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        const position = this.Root.getAttribute('data-district-location');
        const healthValue = this.Root.getAttribute('data-district-health');
        const id = this.Root.getAttribute('data-district-id');
        this._componentID = ComponentID.fromString(id);
        const location = PlotCoord.fromString(position);
        const player = Players.get(this.componentID.owner);
        if (!player) {
            console.error(`district-health.ts: Couldn't find district with ComponentID ${ComponentID.toLogString(this.componentID)}`);
            return;
        }
        const districtId = MapCities.getDistrict(location.x, location.y);
        if (!districtId) {
            console.error(`district-health: couldn't find district ID at hex (${location.x}, ${location.y})`);
            return;
        }
        const district = Districts.get(districtId);
        if (!district || !ComponentID.isValid(districtId)) {
            console.error(`district-health: couldn't find any district with the given id: ${districtId}`);
            return;
        }
        this.isCityCenter = (district.type == DistrictTypes.CITY_CENTER);
        const hslot = document.createElement("fxs-hslot");
        hslot.classList.add("district-health-hslot", "w-full", "h-9");
        this.civHexOuter = document.createElement("div");
        this.civHexOuter.classList.add("bg-contain", "bg-center", "bg-no-repeat", "relative", "w-9", "h-9");
        this.civHexOuter.style.backgroundImage = "url('blp:city_hex_base')";
        this.civHexOuter.style.fxsBackgroundImageTint = UI.Player.getSecondaryColorValueAsString(this.componentID.owner);
        this.civHexInner = document.createElement("div");
        this.civHexInner.classList.add("bg-contain", "bg-center", "bg-no-repeat", "absolute", "inset-1");
        this.civHexInner.style.backgroundImage = "url('blp:city_hex_base')";
        this.civHexInner.style.fxsBackgroundImageTint = UI.Player.getPrimaryColorValueAsString(this.componentID.owner);
        this.civHexOuter.appendChild(this.civHexInner);
        const civIcon = document.createElement("div");
        const iconCSS = Icon.getCivSymbolCSSFromCivilizationType(player.civilizationType);
        civIcon.classList.add("district-health_civ-icon", "absolute", "inset-2", "bg-contain", "bg-center", "bg-no-repeat");
        civIcon.style.backgroundImage = iconCSS;
        civIcon.style.fxsBackgroundImageTint = UI.Player.getSecondaryColorValueAsString(this.componentID.owner);
        this.civHexOuter.appendChild(civIcon);
        hslot.appendChild(this.civHexOuter);
        this.progressBar = document.createElement("div");
        this.progressBar.classList.add("district-health-bar", "relative", "self-center");
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.progressBar.style.backgroundColor = UI.Player.getSecondaryColorValueAsString(this.componentID.owner);
        this.progressBar.style.border = `0.15rem solid ${UI.Player.getSecondaryColorValueAsString(this.componentID.owner)}`;
        this.progressBar.style.borderRadius = '0.66rem';
        this.progressBar.style.boxShadow = "0px 0px 22px rgba(0, 0, 0, 0.70)";
        this.progressBar.style.height = '0.75rem';
        this.progressBar.style.width = '3.66rem';
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.progressBar.setAttribute('value', healthValue);
        this.progressInk = document.createElement("div");
        this.progressInk.style.borderRadius = '0.66rem';
        this.progressInk.style.transition = 'transform 0.5s ease-in-out';
        this.progressInk.classList.add("district-health-bar-ink", "absolute", "inset-0");
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.progressInk.style.backgroundColor = UI.Player.getPrimaryColorValueAsString(this.componentID.owner);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        const healthAmt = parseFloat(healthValue);
        this.progressInk.style.widthPERCENT = healthAmt * 100;
        if (healthAmt <= this.MEDIUM_HEALTH_THRESHHOLD) {
            if (healthAmt <= this.LOW_HEALTH_THRESHHOLD) {
                this.progressInk.classList.add("district-health-low");
            }
            else {
                this.progressInk.classList.add("district-health-med");
            }
        }
        this.progressBar.appendChild(this.progressInk);
        hslot.appendChild(this.progressBar);
        this.Root.appendChild(hslot);
        this.makeWorldAnchor(location);
        // Let the manage know of it's existance; which assigns it an ID.
        const manager = DistrictHealthManager.instance;
        manager.addChildForTracking(this);
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.Root.style.pointerEvents = 'none';
        hslot.style.pointerEvents = 'none';
        this.progressBar.style.pointerEvents = 'none';
        this.hslot = hslot;
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
    }
    onDetach() {
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        window.removeEventListener('interface-mode-changed', this.interfaceModeChangedListener);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.cleanup();
        super.onDetach();
    }
    cleanup() {
        this.destroyWorldAnchor();
        const manager = DistrictHealthManager.instance;
        manager.removeChildFromTracking(this);
    }
    makeWorldAnchor(location) {
        this.destroyWorldAnchor();
        let worldAnchorHandle = null;
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        const DISTRICT_HEALTHBAR_OFFSET = { x: 0, y: 0, z: 0 };
        /*
        const units = MapUnits.getUnits(location.x, location.y);
        const numUnits = units.length;
        if (numUnits > 1) {
            DISTRICT_HEALTHBAR_OFFSET.x += 10;
        }
        if (this.isCityCenter) {
            DISTRICT_HEALTHBAR_OFFSET.y -= 25;
        }
         */
        worldAnchorHandle = WorldAnchors.RegisterFixedWorldAnchor(location, DISTRICT_HEALTHBAR_OFFSET);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        if (worldAnchorHandle !== null && worldAnchorHandle >= 0) {
            this.Root.setAttribute('data-bind-style-transform2d', `{{FixedWorldAnchors.offsetTransforms[${worldAnchorHandle}].value}}`);
            this.Root.setAttribute('data-bind-style-opacity', `{{FixedWorldAnchors.visibleValues[${worldAnchorHandle}]}}`);
        }
        else {
            console.error(`Failed to create WorldAnchorHandle for DistrictHealthBar, District id: ${ComponentID.toLogString(this._componentID)}`);
        }
    }
    destroyWorldAnchor() {
        if (this._worldAnchorHandle) {
            this.Root.removeAttribute('data-bind-style-transform2d');
            this.Root.removeAttribute('data-bind-style-opacity');
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            WorldAnchors.UnregisterFixedWorldAnchor(this._worldAnchorHandle);
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
        this._worldAnchorHandle = null;
    }
    setContested(_isContested, controllingPlayer) {
        const player = Players.get(controllingPlayer);
        if (player) {
            if (this.civHexOuter && this.civHexInner) {
                const civIcon = MustGetElement(".district-health_civ-icon", this.civHexOuter);
                const iconCSS = Icon.getCivSymbolCSSFromCivilizationType(player.civilizationType);
                civIcon.style.backgroundImage = iconCSS;
                civIcon.style.fxsBackgroundImageTint = UI.Player.getSecondaryColorValueAsString(controllingPlayer);
                this.civHexInner.style.fxsBackgroundImageTint = UI.Player.getPrimaryColorValueAsString(controllingPlayer);
                this.civHexOuter.style.fxsBackgroundImageTint = UI.Player.getSecondaryColorValueAsString(controllingPlayer);
            }
        }
        else {
            console.warn(`district-health: couldn't find player ID ${controllingPlayer} in setContested()`);
        }
    }
    updateDistrictHealth(value) {
        if (this.progressBar && this.progressInk) {
            const healthAmt = parseFloat(value);
            this.progressInk.style.widthPERCENT = healthAmt * 100;
            this.progressInk.classList.remove("district-health-med");
            this.progressInk.classList.remove("district-health-low");
            if (healthAmt <= this.MEDIUM_HEALTH_THRESHHOLD) {
                if (healthAmt <= this.LOW_HEALTH_THRESHHOLD) {
                    this.progressInk.classList.add("district-health-low");
                }
                else {
                    this.progressInk.classList.add("district-health-med");
                }
            }
        }
    }
    get componentID() {
        return this._componentID;
    }
}
Controls.define('district-health-bar', {
    createInstance: DistrictHealthBar,
    description: 'District Health Bar',
    classNames: ['district-health-container'],
    styles: ["fs://game/base-standard/ui/district/district-health.css"],
});

//# sourceMappingURL=file:///base-standard/ui/district/district-health.js.map
