/**
 * @file unit-flags-independent-powers.ts
 * @copyright 2021-2023, Firaxis Games
 * @description Unit flag for independent powers.  These eventually can become city-states.
 */
import { utils } from '/core/ui/graph-layout/utils.js';
import { ComponentID } from '/core/ui/utilities/utilities-component-id.js';
import { Icon } from '/core/ui/utilities/utilities-image.js';
import { UnitFlagFactory, UnitFlagManager } from '/base-standard/ui/unit-flags/unit-flag-manager.js';
import { Layout } from '/core/ui/utilities/utilities-layout.js';
import { ActionActivateEvent } from '/core/ui/components/fxs-activatable.js';
// ------------------ MODIFIED BY NASU ------------------ START ------------------
import { getSettings } from '/scripts/nasuellia-alt-unit-flags-and-bars-settings.js';
import { animationClock } from '/scripts/war-icon-sync-clock.js';

const hasPulseGlow = document.querySelector('style[data-pulse-glow]');
if (!hasPulseGlow) {
    const styleSheet = document.createElement('style');
    styleSheet.setAttribute('data-pulse-glow', 'true');
    styleSheet.textContent = `@keyframes pulseGlow {0% { transform: scale(1.00); opacity: 0.5; } 50% { transform: scale(1.15); opacity: 0.80; } 100% { transform: scale(1.00); opacity: 0.5; }}`;
    document.head.appendChild(styleSheet);
}

const getCurrentSettings = () => getSettings("nasuelliaAltUnitFlagsAndBarsSettingsKey");
function manipulateColor(color, percent) {
    const rgb = color.match(/\d+/g)?.map(Number);
    if (!rgb || rgb.length < 3) {
        return color;
    }
    const darkened = rgb.map(value => Math.round(value * (1 - percent)));
    return `rgb(${darkened[0]}, ${darkened[1]}, ${darkened[2]})`;
}
class DebugUtils {
    static inspectObject(obj, label = "Object") {
        if (!obj) {
            console.warn(`NASUDEBUG: ${label} is null or undefined`);
            return;
        }

        console.warn(`NASUDEBUG: ${label} exists, type:`, typeof obj);

        const ownProps = Object.getOwnPropertyNames(obj);
        console.warn(`NASUDEBUG: ${label} own properties count:`, ownProps.length);
        if (ownProps.length > 0) {
            ownProps.forEach(prop => {
                const value = obj[prop];
                const valueType = typeof value;
                const paramCount = valueType === 'function' ? `(${value.length} params)` : '';
                console.warn(`NASUDEBUG: ${label} own property: ${prop} (${valueType}${paramCount})`, value);
            });
        } else {
            console.warn(`NASUDEBUG: No own properties found in ${label}`);
        }

        const proto = Object.getPrototypeOf(obj);
        if (proto) {
            const protoProps = Object.getOwnPropertyNames(proto);
            console.warn(`NASUDEBUG: ${label} prototype properties count:`, protoProps.length);
            if (protoProps.length > 0) {
                protoProps.forEach(prop => {
                    const value = proto[prop];
                    const valueType = typeof value;
                    const paramCount = valueType === 'function' ? `(${value.length} params)` : '';
                    console.warn(`NASUDEBUG: ${label} proto property: ${prop} (${valueType}${paramCount})`, value);
                });
            } else {
                console.warn(`NASUDEBUG: No prototype properties found in ${label}`);
            }
        } else {
            console.warn(`NASUDEBUG: ${label} has no prototype`);
        }
    }
}
// ------------------ MODIFIED BY NASU ------------------ END ------------------

/**
 * Create independent powers (IP) flags which are used for city-states as well.
 * Also maintains IP specific states for changes.
 */
class IndependentPowersFlagMaker {
    initialize() {
        engine.on('DiplomacyEventStarted', (data) => { IndependentPowersFlagMaker.onDiplomacyEventStarted(data); });
        engine.on('DiplomacyEventEnded', (data) => { IndependentPowersFlagMaker.onDiplomacyEventEnded(data); });
    }
    isMatch(unit, _unitDefinition, _others) {
        const playerID = unit.owner;
        const player = Players.get(playerID);
        if (player?.isIndependent) {
            return true;
        }
        return false;
    }
    getComponentName() {
        return "unit-flags-independent-powers";
    }
    static addChildForTracking(index, flag) {
        let flagsOfIndex = this.ipflags.get(index);
        if (flagsOfIndex == undefined) {
            this.ipflags.set(index, [flag]); // First flag of an index, add it.
            return;
        }
        // If here at least one flag of the index type has been created; make sure it isn't a dupe and add.
        const existingFlag = flagsOfIndex.find(existingFlag => { return ComponentID.isMatch(existingFlag.componentID, flag.componentID); });
        if (existingFlag != undefined) {
            console.error(`unit-flags-independent-powers: Attempt to add a IP unit flag with index ${index} for factory tracking but its already being tracked. cid: ${ComponentID.toLogString(flag.componentID)}`);
            return;
        }
        flagsOfIndex.push(flag);
    }
    static removeChildFromTracking(index, flag) {
        let flagsOfIndex = this.ipflags.get(index);
        if (flagsOfIndex == undefined) {
            console.warn(`unit-flags-independent-powers: Attempt to remove child from tracking at factory but no children of that index '${index}' exist.`);
            return;
        }
        const found = flagsOfIndex.some((existingFlag, index, array) => {
            if (ComponentID.isMatch(existingFlag.componentID, flag.componentID)) {
                array.splice(index, 1);
                return true;
            }
            return false;
        });
        if (!found) {
            console.warn(`unit-flags-independent-powers: Was unable to find flag to delete from factory. index: ${index}, unit: ${ComponentID.toLogString(flag.componentID)}`);
        }
    }
    static onDiplomacyEventStarted(_data) {
        // TODO: use data.location to match up index or type and update only those flags (current updates all)
        IndependentPowersFlagMaker.ipflags.forEach((flagArray) => {
            flagArray.some((flag) => {
                flag.updateAffinity();
            });
        });
    }
    static onDiplomacyEventEnded(data) {
        if (data.location == undefined) {
            return; // Different diplomacy event, not related to independent powers (affinity changing)
        }
        const playerID = GameplayMap.getOwner(data.location.x, data.location.y);
        const player = Players.get(playerID);
        if (!player) {
            return; // Valid for some diplomacy events: No one owns the plot location.
        }
        const independentID = Game.IndependentPowers.getIndependentPlayerIDAt(data.location.x, data.location.y);
        if (independentID == PlayerIds.NO_PLAYER) {
            return; // Valid for events not related to independents.
        }
        const flagArray = IndependentPowersFlagMaker.ipflags.get(independentID);
        if (flagArray == undefined) {
            return; // No flag for that type exist.
        }
        flagArray.some((flag) => {
            flag.updateAffinity();
        });
    }
}
IndependentPowersFlagMaker.ipflags = new Map();
export class IndependentPowersUnitFlag extends Component {
    constructor() {
        super(...arguments);
        this._componentID = ComponentID.getInvalidID();
        this._worldAnchor = null;
        this.engineInputListener = (inputEvent) => { this.onEngineInput(inputEvent); };
        this.unitContainer = null;
        this.unitHealthBar = null;
        this.unitHealthBarInner = null;
        this.unitFlagIcon = null;
        this.unitFlagPromotionNumber = null;
        this.isHidden = false;
        this.independentID = PlayerIds.NO_PLAYER;
        /**
         * A vertical offset when the unit is 'stacked' with other units.
         * TODO - The unit world anchor should be able to incorporate this offset in C++ to avoid constantly recalculating this in Script.
         */
        this.flagOffset = 0;
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.unitHealthBarContainer = null;
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
    }


// ------------------ MODIFIED BY NASU ------------------ START ------------------
    updateArmy() {
        if (this.unit.isCommanderUnit) {
            this.realizePromotions();
            this.updateArmyDots();
        }
    }
    // ------------------ MODIFIED BY NASU ------------------ END ------------------
    onAttach() {
        super.onAttach();
        const id = this.Root.getAttribute('unit-id');
        this._componentID = ComponentID.fromString(id);
        // Obtain player colors used later on.
        let playerColorPri = 'rgb(0, 0, 0)';
        let playerColorSec = 'rgb(255, 255, 255)';
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        const unit = Units.get(this.componentID);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        // Flag Container
        const unitFlagContainer = document.createElement('div');
        unitFlagContainer.classList.add('unit-flag__container', 'absolute', '-top-6', '-left-6', 'pointer-events-auto', 'flex', 'flex-col', 'justify-center', 'items-center', 'w-12', 'h-12');
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        unitFlagContainer.style.width = '3.75rem';
        unitFlagContainer.style.height = '3.75rem';
        unitFlagContainer.style.transition = 'transform 0.3s ease-in-out';
        this.unitContainer = unitFlagContainer;
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        // Flag Shadow
        const unitFlagShadow = document.createElement('div');
        unitFlagShadow.classList.add('unit-flag__shadow', 'pointer-events-none', 'absolute', 'inset-0', 'bg-cover');
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        unitFlagShadow.style.filter = 'blur(14px) opacity(33%)';
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        unitFlagContainer.appendChild(unitFlagShadow);
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        // Hostility border
        const unitflagHostileOutline = document.createElement('div');
        unitflagHostileOutline.classList.add('unit-flag__hostile-outline', 'absolute', 'pointer-events-none');
        unitflagHostileOutline.style.filter = 'blur(12px)';
        unitflagHostileOutline.style.backgroundColor = playerColorSec;
        unitflagHostileOutline.style.animation = 'pulseGlow 1.5s ease-in-out infinite';
        unitFlagContainer.appendChild(unitflagHostileOutline);
        /*
        if (this.componentID.owner == GameContext.localObserverID) {
            // Vanilla Broken Highlight
            const unitFlagHighlight = document.createElement('div');
            unitFlagHighlight.classList.add('unit-flag__highlight', 'opacity-0', 'pointer-events-none', 'absolute', 'bg-no-repeat');
            unitFlagContainer.appendChild(unitFlagHighlight);
        }
         */
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        // Inner Shape
        const unitFlagInnerShape = document.createElement('div');
        unitFlagInnerShape.classList.add('unit-flag__shape', 'unit-flag__shape--inner', "pointer-events-none", "absolute", "inset-0", "bg-no-repeat");
        unitFlagInnerShape.style.fxsBackgroundImageTint = playerColorPri;
        unitFlagContainer.appendChild(unitFlagInnerShape);
        // Outer Shape
        const unitFlagOuterShape = document.createElement('div');
        unitFlagOuterShape.classList.add('unit-flag__shape', 'unit-flag__shape--outer', "pointer-events-none", "absolute", "inset-0", "bg-no-repeat");
        unitFlagOuterShape.style.fxsBackgroundImageTint = playerColorSec;
        unitFlagContainer.appendChild(unitFlagOuterShape);
        // Healthbar Container
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.unitHealthBarContainer = document.createElement('div');
        this.unitHealthBarContainer.classList.add('unit-flag__healthbar-container', 'absolute', 'h-full', 'self-center', 'pointer-events-none');
        this.unitHealthBarContainer.style.width = '2.2rem';
        this.adjustHealthbarPosition();
        unitFlagContainer.appendChild(this.unitHealthBarContainer);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        // Healthbar Outer
        const unitFlagHealthbar = document.createElement('div');
        unitFlagHealthbar.classList.add('unit-flag__healthbar', 'absolute', 'h-1', 'bg-black');
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        unitFlagHealthbar.style.height = '0.700rem';
        unitFlagHealthbar.style.border = `0.20rem solid ${playerColorPri}`;
        unitFlagHealthbar.style.backgroundColor = playerColorPri;
        unitFlagHealthbar.style.borderRadius = '0.66rem';
        unitFlagHealthbar.style.boxShadow = "0px 0px 20px rgba(0, 0, 0, 0.9)";
        this.unitHealthBarContainer.appendChild(unitFlagHealthbar);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.unitHealthBar = unitFlagHealthbar;
        // Healthbar Inner
        const unitFlagHealthbarInner = document.createElement('div');
        unitFlagHealthbarInner.classList.add('unit-flag__healthbar-inner', 'absolute', 'h-full', 'bg-no-repeat');
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        unitFlagHealthbarInner.style.borderRadius = '0.66rem';
        unitFlagHealthbarInner.style.backgroundColor = playerColorSec;
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        unitFlagHealthbar.appendChild(unitFlagHealthbarInner);
        this.unitHealthBarInner = unitFlagHealthbarInner;
        // Icon
        const unitFlagIcon = document.createElement('div');
        unitFlagIcon.classList.add('unit-flag__icon', 'pointer-events-none', 'absolute', 'bg-contain', 'bg-no-repeat');
        unitFlagIcon.style.fxsBackgroundImageTint = playerColorSec;
        this.unitFlagIcon = unitFlagIcon;
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        unitFlagContainer.appendChild(this.unitFlagIcon);
        this.adjustUnitIconPosition(false);
        // Promotion / Tier number
        const unitFlagPromotionNumber = document.createElement('div');
        unitFlagPromotionNumber.classList.add('unit-flag__promotion-number', 'font-body', 'text-2xs', 'absolute', 'text-center');
        unitFlagPromotionNumber.style.color = playerColorSec;
        unitFlagPromotionNumber.style.left = '50%';
        unitFlagPromotionNumber.style.top = '2.45rem';
        unitFlagPromotionNumber.style.marginLeft = '-0.55rem';
        unitFlagPromotionNumber.style.fontWeight  = "800";
        unitFlagPromotionNumber.style.fontSize = '0.55rem';
        unitFlagPromotionNumber.style.width = '1.05rem';
        unitFlagPromotionNumber.style.height = '1.05rem';
        unitFlagPromotionNumber.style.lineHeight = '1.00rem';
        unitFlagPromotionNumber.style.backgroundColor = manipulateColor(playerColorPri, 0.15);
        unitFlagPromotionNumber.style.border = `0.07rem solid ${playerColorSec}`;
        unitFlagPromotionNumber.style.borderRadius = '50%';
        unitFlagContainer.appendChild(unitFlagPromotionNumber);
        this.unitFlagPromotionNumber = unitFlagPromotionNumber;
        // Army Stats
        /*
        const unitFlagArmyStats = document.createElement('div');
        unitFlagArmyStats.classList.add('unit-flag__army-stats"', 'items-center', 'text-center', 'absolute', '-left-3', '-right-3', 'bg-transparent');
        unitFlagContainer.appendChild(unitFlagArmyStats);
         */
        // Promotion / Tier number (vanilla)
        /*
        const unitFlagPromotionContainer = document.createElement('div');
        unitFlagPromotionContainer.classList.add('promotion-container', 'flex', 'flex-col-reverse', 'pointer-events-none', 'absolute');
        unitFlagContainer.appendChild(unitFlagPromotionContainer);
         */
        // Army Dots
        const unitFlagLeftDots = document.createElement('div');
        unitFlagLeftDots.classList.add('unit-flag__left-dots', 'absolute', 'flex', 'flex-col-reverse', 'justify-end');
        this.unitFlagLeftDots = unitFlagLeftDots;
        const unitFlagRightDots = document.createElement('div');
        unitFlagRightDots.classList.add('unit-flag__right-dots', 'absolute', 'flex', 'flex-col-reverse', 'justify-end');
        unitFlagContainer.appendChild(unitFlagRightDots);

        if (unit && unit.isFleetCommander || unit && unit.isSquadronCommander  || unit && unit.isAerodromeCommander) {
            unitFlagLeftDots.style.top = '0.65rem';
            unitFlagLeftDots.style.left = '0.70rem';
            unitFlagRightDots.style.top = '0.65rem';
            unitFlagRightDots.style.right = '0.70rem';
        }
        if (unit && unit.isArmyCommander) {
            unitFlagLeftDots.style.top = '0.85rem';
            unitFlagLeftDots.style.left = '0.45rem';
            unitFlagLeftDots.style.transform = 'rotate(-45deg)';
            unitFlagRightDots.style.top = '0.85rem';
            unitFlagRightDots.style.right = '0.45rem';
            unitFlagRightDots.style.transform = 'rotate(45deg)';
        }
        unitFlagContainer.appendChild(unitFlagLeftDots);
        this.unitFlagRightDots = unitFlagRightDots;
        // Logic for icon swaps, etc
        this.activeStatusIcons = new Map();
        this.originalUnitIcon = null;
        this.initializeArmyDots();
        this.updateMovement();
        this.updateArmyDots();
        engine.on('DiplomacyEventStarted', this.onDiplomacyEventStarted, this);
        engine.on('DiplomacyEventEnded', this.onDiplomacyEventEnded, this);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        // Finalize
        this.Root.appendChild(unitFlagContainer);
        engine.on('AffinityLevelChanged', this.onAffinityLevelChanged, this);
        engine.on('BeforeUnload', this.onUnload, this);
        const manager = UnitFlagManager.instance;
        manager.addChildForTracking(this);
        // Sanity check early on (subsequent unit calls will be from componentID in getter)
        if (!unit) {
            console.error("unit-flags-independent-powers: Could not attach unit flag; no unit object for cid: ", ComponentID.toLogString(this.componentID));
            return;
        }
        // Obtain independent index used in various realization calls.
        const playerID = this.componentID.owner;
        const player = Players.get(playerID);
        if (!player) {
            console.error(`unit-flags-independent-powers: failed to get player of independent in attaching flag. palyerID: ${playerID} for ${ComponentID.toLogString(this.componentID)}.`);
            return;
        }
        this.independentID = Game.IndependentPowers.getIndependentPlayerIDFromUnit(this.componentID);
        IndependentPowersFlagMaker.addChildForTracking(this.independentID, this); // Register with factory to IPflag specific updates
        this.Root.addEventListener('engine-input', this.engineInputListener);
        this.makeWorldAnchor(this.componentID);
        this.realizeIcon();
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.updateAffinity();
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.realizeUnitHealth();
        this.realizePromotions();
        this.realizeTooltip();
        if (unit) {
            const location = unit.location;
            const revealedState = GameplayMap.getRevealedState(GameContext.localObserverID, location.x, location.y);
            if (!this.isHidden) {
                this.setVisibility(revealedState);
            }
        }
        else {
            this.setVisibility(RevealedStates.HIDDEN);
        }
        this.checkUnitPosition(unit);
    }

    // ------------------ MODIFIED BY NASU ------------------ START ------------------
    onDiplomacyEventStarted(data) {
        const actionType = data.actionType;
        if (actionType === -434781988) {
            setTimeout(() => {
                this.updateAffinity();
            }, 250);
        }
    }
    onDiplomacyEventEnded(data) {
        const actionType = data.actionType;
        if (actionType === -434781988) {
            setTimeout(() => {
                this.updateAffinity();
            }, 250);
        }
    }
    // ------------------ MODIFIED BY NASU ------------------ END ------------------
    onUnload() {
        this.cleanup();
    }
    onDetach() {
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        engine.off('DiplomacyEventStarted', this.onDiplomacyEventStarted, this);
        engine.off('DiplomacyEventEnded', this.onDiplomacyEventStarted, this);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.cleanup();
        super.onDetach();
    }
    cleanup() {
        const manager = UnitFlagManager.instance;
        IndependentPowersFlagMaker.removeChildFromTracking(this.independentID, this);
        manager.removeChildFromTracking(this);
        engine.off('AffinityLevelChanged', this.onAffinityLevelChanged, this);
        engine.off('BeforeUnload', this.onUnload, this);
        this.destroyWorldAnchor();
        this.Root.removeEventListener('engine-input', this.engineInputListener);
        this._componentID = ComponentID.getInvalidID();
    }
    onEngineInput(inputEvent) {
        if (inputEvent.detail.status != InputActionStatuses.FINISH) {
            return;
        }
        if (inputEvent.detail.name == "accept" || inputEvent.detail.name == "mousebutton-left" || inputEvent.detail.name == "touch-tap") {
            if (ComponentID.isInvalid(this._componentID)) {
                console.warn("unit-flags-independent-powers: Attempt to activate a unit-flag but invalid associated unit.");
                return;
            }
            if (GameContext.localObserverID != this._componentID.owner) {
                return;
            }
            UI.Player.selectUnit(this._componentID);
            window.dispatchEvent(new SetActivatedComponentEvent(null));
            this.Root.dispatchEvent(new ActionActivateEvent(inputEvent.detail.x, inputEvent.detail.y));
            inputEvent.stopPropagation();
            inputEvent.preventDefault();
        }
    }
    hide() {
        this.isHidden = true;
        this.unitContainer?.classList.add("hidden");
    }
    show() {
        this.isHidden = false;
        if (this.unit) {
            const location = this.unit.location;
            const revealedState = GameplayMap.getRevealedState(GameContext.localObserverID, location.x, location.y);
            this.setVisibility(revealedState);
        }
    }
    disable() {
        this.unitContainer?.classList.add("disabled");
    }
    enable() {
        this.unitContainer?.classList.remove("disabled");
    }
    /**
     * @description An independent power's affinity level with a player changed.  Update flags if it's the local player.
     * @param {AffinityLevelChanged_EventData} data
     */
    onAffinityLevelChanged(data) {
        if (data.player == GameContext.localObserverID) {
            this.updateAffinity();
        }
    }
    /**
     * Helper to get the Indy object related to this flag.
     * @returns {IndependentDefition|null}
     */
    getIndyName() {
        const name = Game.IndependentPowers.independentName(this.independentID);
        if (name != null) {
            return name;
        }
        return "";
    }
    realizeUnitHealth() {
        if (!this.unitHealthBar) {
            console.error("unit-flags-independent-powers: realizeUnitHealth(): Missing this.unitHealthBar with '.unit-flag__healthbar'. cid: " + ComponentID.toLogString(this.componentID));
            return;
        }
        let unit = this.unit;
        // reset health stats
        this.unitHealthBar.classList.remove("unit-flag__healthbar-med-health");
        this.unitHealthBar.classList.remove("unit-flag__healthbar-low-health");
        let damage = 1.0;
        if (unit?.Health) {
            damage = (unit.Health.maxDamage - unit.Health.damage) / unit.Health.maxDamage;
            if (unit.Health.damage > 0 || getCurrentSettings().HealthbarVisibility === 'permanent') {
                this.unitContainer?.classList.toggle("unit-flag--with-healthbar", true);
            }
            else {
                this.unitContainer?.classList.toggle("unit-flag--with-healthbar", false);
            }
            /*
            this.unitContainer?.classList.toggle("unit-flag--with-healthbar", (unit.Health.damage > 0));
            if (damage <= 0.75 && damage >= 0.5) {
                this.unitContainer?.classList.add("unit-flag__healthbar-med-health");
            }
            else if (damage < 0.5) {
                this.unitContainer?.classList.add("unit-flag__healthbar-low-health");
            }
             */
        }
        if (this.unitHealthBarInner) {
            this.unitHealthBarInner.style.widthPERCENT = utils.clamp(damage, 0, 1) * 100;
        }
    }
    realizeIcon() {
        const unit = this.unit;
        if (!unit) {
            console.error("unit-flags-independent-powers: Unit flag finished loading its HTML content but is not associated with a valid unit. cid: ", ComponentID.toLogString(this.componentID));
            return;
        }
        const unitDefinition = GameInfo.Units.lookup(unit.type);
        if (!unitDefinition) {
            console.warn("unit-flags-independent-powers: Cannot set unit flag icon due to missing Unit Definition. type: ", unit.type, "  cid: ", ComponentID.toLogString(this.componentID));
            return;
        }
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        if (unit.isArmyCommander) {
            this.unitContainer?.classList.add('unit-flag--army');
        }
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        else if (unit.Combat?.canAttack) {
            this.unitContainer?.classList.add('unit-flag--combat');
        }
        else if (unitDefinition.CoreClass != "CORE_CLASS_SUPPORT" && unitDefinition.CoreClass != "CORE_CLASS_RECON") {
            this.unitContainer?.classList.add('unit-flag--civilian');
        }
        if (this.unitFlagIcon) {
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            const iconName = Icon.getUnitIconFromDefinition(unitDefinition);
            this.originalUnitIcon = iconName;
            this.unitFlagIcon.style.backgroundImage = `url(${iconName})`;
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
    }
    realizeTooltip() {
        const playerId = this.componentID.owner;
        const player = Players.get(playerId);
        if (player) {
            const unit = this.unit;
            const unitName = ((unit) ? Locale.compose(unit.name) : "ERROR, unit: " + ComponentID.toLogString(this._componentID));
            const playerName = Locale.compose("LOC_UNITFLAG_INDEPENDENT_POWER_NAME", this.getIndyName());
            const affinityRelationship = Locale.compose(Game.IndependentPowers.getIndependentHostility(this.independentID, GameContext.localObserverID));
            const tooltipDiv = this.Root.querySelector('.unit-flag__container');
            if (tooltipDiv) {
                tooltipDiv.setAttribute("data-tooltip-content", `<div>${playerName}</div><div>${unitName}</div><div>${affinityRelationship}</div>`);
            }
        }
    }

    // ------------------ MODIFIED BY NASU ------------------ START ------------------
    realizePromotions() {
        const unitDefinition = GameInfo.Units.lookup(this.unit.type);
        if (!this.unit || !unitDefinition) {
            console.warn("unit-flag: Cannot set promotions due to missing Unit Definition. type: ", this.unit.type, "  cid: ", ComponentID.toLogString(this.componentID));
            return;
        }
        if (this.unit.isCommanderUnit) {
            while (this.unitFlagPromotionNumber.hasChildNodes()) {
                this.unitFlagPromotionNumber.removeChild(this.unitFlagPromotionNumber.lastChild);
            }
            const numPromotions = this.unit.Experience?.getLevel;
            const promotionNumber = document.createElement("div");
            promotionNumber.classList.add("w-4", "h-4");
            promotionNumber.innerHTML = numPromotions.toString();
            this.unitFlagPromotionNumber.appendChild(promotionNumber);
            this.Root.classList.add("unit-flag--has-promotions");

        }
        else if (unitDefinition.CoreClass == "CORE_CLASS_MILITARY") {
            while (this.unitFlagPromotionNumber.hasChildNodes()) {
                this.unitFlagPromotionNumber.removeChild(this.unitFlagPromotionNumber.lastChild);
            }
            const numPromotions = unitDefinition.Tier;
            const numPromotionsRoman = numPromotions > 0 && numPromotions <= 10 ? ['I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX', 'X'][numPromotions - 1] : '';
            const promotionNumber = document.createElement("div");
            promotionNumber.classList.add("w-4", "h-4");
            promotionNumber.innerHTML = numPromotionsRoman;
            this.unitFlagPromotionNumber.appendChild(promotionNumber);
            this.Root.classList.add("unit-flag--has-promotions");
        }
        else {
            this.unitFlagPromotionNumber.style.visibility = 'hidden';
        }
    }
    // ------------------ MODIFIED BY NASU ------------------ END ------------------
    
    /**
     * Change the visibility of the unit's flag.
     * @param {RevealState} state - The visibility state to change to.
     */
    setVisibility(state) {
        if (this.isHidden) {
            return;
        }
        switch (state) {
            case RevealedStates.HIDDEN:
                this.unitContainer?.classList.add("hidden");
                break;
            case RevealedStates.REVEALED:
                this.unitContainer?.classList.add("hidden");
                break;
            case RevealedStates.VISIBLE:
                this.unitContainer?.classList.remove("hidden");
                break;
            default:
                console.warn("unit-flags-independent-powers: Unknown visibility reveal type passed to unit flag. vis: ", state, "  cid: ", ComponentID.toLogString(this.componentID));
                break;
        }
    }
    makeWorldAnchor(componentID) {
        this.destroyWorldAnchor();
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        const height = 42.0;
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        const worldAnchor = WorldAnchors.RegisterUnitAnchor(componentID, height);
        if (worldAnchor) {
            this.Root.setAttribute('data-bind-style-transform2d', `{{UnitAnchors.offsetTransforms[${worldAnchor}].value}}`);
            this.Root.setAttribute('data-bind-style-opacity', `{{UnitAnchors.visibleValues[${worldAnchor}]}}`);
        }
        else {
            console.error(`Failed to create WorldAnchor for unit ${JSON.stringify(componentID)}.`);
        }
    }
    destroyWorldAnchor() {
        if (this._worldAnchor) {
            this.Root.removeAttribute('data-bind-style-transform2d');
            this.Root.removeAttribute('data-bind-style-opacity');
            WorldAnchors.UnregisterUnitAnchor(this._worldAnchor);
        }
        this._worldAnchor = null;
    }
    updateHealth() {
        this.realizeUnitHealth();
    }
    
    // ------------------ MODIFIED BY NASU ------------------ START ------------------
    initializeArmyDots() {
        this.leftDots = [];
        this.rightDots = [];

        const playerColorSec = UI.Player.getSecondaryColorValueAsString(this.componentID.owner);
        const playerColorPri = UI.Player.getPrimaryColorValueAsString(this.componentID.owner);

        for (let i = 0; i < 4; i++) {
            const dot = document.createElement('div');
            dot.classList.add('unit-flag__dot');
            dot.style.width = '0.45rem';
            dot.style.height = '0.45rem';
            dot.style.borderRadius = '50%';
            dot.style.border = `1px solid ${playerColorPri}`;
            dot.style.backgroundColor = playerColorSec;
            dot.style.marginTop = '0.05rem';
            dot.style.visibility = 'hidden';
            this.unitFlagLeftDots.appendChild(dot);
            this.leftDots.push(dot);
        }

        for (let i = 0; i < 4; i++) {
            const dot = document.createElement('div');
            dot.classList.add('unit-flag__dot');
            dot.style.width = '0.45rem';
            dot.style.height = '0.45rem';
            dot.style.borderRadius = '50%';
            dot.style.border = `1px solid ${playerColorPri}`;
            dot.style.backgroundColor = playerColorSec;
            dot.style.marginTop = '0.05rem';
            dot.style.visibility = 'hidden';
            this.unitFlagRightDots.appendChild(dot);
            this.rightDots.push(dot);
        }
    }
    updateArmyDots() {
        let unitCount = 0;
        if (this.unit.isCommanderUnit) {
            const army = this.unit.armyId ? Armies.get(this.unit.armyId) : null;
            unitCount = army.unitCount;
        }
        
        unitCount = Math.min(Math.max(unitCount, 0), 8);
        
        this.leftDots.forEach(dot => dot.style.visibility = 'hidden');
        this.rightDots.forEach(dot => dot.style.visibility = 'hidden');
        
        for (let i = 0; i < unitCount; i++) {
            if (i % 2 === 0) {
                const leftIndex = Math.floor(i / 2);
                if (leftIndex < 4) {
                    this.leftDots[leftIndex].style.visibility = 'visible';
                }
            } else {
                const rightIndex = Math.floor(i / 2);
                if (rightIndex < 4) {
                    this.rightDots[rightIndex].style.visibility = 'visible';
                }
            }
        }
    }

    startHostileIconSwap() {
        if (!this.unitFlagIcon || !this.originalUnitIcon) return;
        if (this.iconSwapSubscription) return;
        this.unitFlagIcon.style.transition = 'transform 0.1s ease'; // Default for initial shrink

        var updateIcon = function() {
            var phase = animationClock.getPhase();
            if (phase === 'grow') {
                this.unitFlagIcon.style.transform = 'scale(0.8)'; // Shrink before skull
                setTimeout(function() {
                    this.unitFlagIcon.style.backgroundImage = 'url(fs://game/nasuellia-alt-unit-flags-and-bars/icons/war.png)';
                    this.adjustUnitIconPosition(true);
                    this.unitFlagIcon.style.transition = 'transform 0.1s ease-out'; // Overshoot
                    this.unitFlagIcon.style.transform = 'scale(1.2)'; // Overshoot past 1.0
                }.bind(this), 100); // Half of grow’s 200ms phase
            } else if (phase === 'skull') {
                this.unitFlagIcon.style.transition = 'transform 0.1s ease-in'; // Settle to 1.0
                this.unitFlagIcon.style.transform = 'scale(1.0)';
                this.unitFlagIcon.style.backgroundImage = 'url(fs://game/nasuellia-alt-unit-flags-and-bars/icons/war.png)';
            } else if (phase === 'shrink') {
                this.unitFlagIcon.style.transform = 'scale(0.8)'; // Shrink before normal
                setTimeout(function() {
                    this.unitFlagIcon.style.backgroundImage = 'url(' + this.originalUnitIcon + ')';
                    this.adjustUnitIconPosition(false);
                    this.unitFlagIcon.style.transition = 'transform 0.1s ease-out'; // Overshoot
                    this.unitFlagIcon.style.transform = 'scale(1.2)'; // Overshoot past 1.0
                }.bind(this), 100); // Half of shrink’s 200ms phase
            } else if (phase === 'normal') {
                this.unitFlagIcon.style.transition = 'transform 0.1s ease-in'; // Settle to 1.0
                this.unitFlagIcon.style.transform = 'scale(1.0)';
                this.unitFlagIcon.style.backgroundImage = 'url(' + this.originalUnitIcon + ')';
            }
        }.bind(this);
        this.iconSwapSubscription = updateIcon;
        animationClock.subscribe(updateIcon);
        animationClock.start();
    }

    stopHostileIconSwap() {
        if (this.iconSwapSubscription) {
            animationClock.unsubscribe(this.iconSwapSubscription);
            this.iconSwapSubscription = null;
            this.unitFlagIcon.style.transition = '';
            this.unitFlagIcon.style.transform = 'scale(1.0)';
            this.unitFlagIcon.style.backgroundImage = 'url(' + this.originalUnitIcon + ')';
            this.adjustUnitIconPosition(false);
            console.warn('NASU: Stopped hostile swap, reset to normal');
        }
    }

    adjustHealthbarPosition(){
        const unit = Units.get(this.componentID);
        if (!unit || !this.unitHealthBarContainer) {
            return;
        }
        const myUnitDefinition = GameInfo.Units.lookup(unit.type);
        if (unit.isFleetCommander || unit.isSquadronCommander) {
            this.unitHealthBarContainer.style.bottom = '0.15rem';
            return;
        }
        if (unit.isArmyCommander) {
            this.unitHealthBarContainer.style.bottom = '0.30rem';
            return;
        }
        if (myUnitDefinition && myUnitDefinition.CoreClass === "CORE_CLASS_MILITARY") {
            this.unitHealthBarContainer.style.bottom = '0.30rem';
            return;
        }
        if (myUnitDefinition && (myUnitDefinition.CoreClass === "CORE_CLASS_RECON" || myUnitDefinition.CoreClass === "CORE_CLASS_SUPPORT")) {
            this.unitHealthBarContainer.style.bottom = '0.20rem';
            return;
        }
        this.unitHealthBarContainer.style.bottom = '0.40rem';
    }

    adjustUnitIconPosition(isWarIcon){
        const unit = Units.get(this.componentID);
        if (!unit || !this.unitFlagIcon) {
            return;
        }
        const myUnitDefinition = GameInfo.Units.lookup(unit.type);
        if (unit.isFleetCommander || unit.isSquadronCommander) {
            this.unitFlagIcon.style.marginTop = '0.5rem';
            return;
        }
        if (unit.isArmyCommander) {
            this.unitFlagIcon.style.marginTop = '-0.175rem';
            this.unitFlagIcon.style.marginLeft = isWarIcon === true ? '0.015rem' : '0.00rem';
            return;
        }
        if (myUnitDefinition && myUnitDefinition.CoreClass === "CORE_CLASS_MILITARY") {
            this.unitFlagIcon.style.marginTop = isWarIcon === true ? '0.15rem' : '0.15rem';
            this.unitFlagIcon.style.marginLeft = isWarIcon === true ? '0.00rem' : '0.00rem';
            return;
        }
        if (myUnitDefinition && (myUnitDefinition.CoreClass === "CORE_CLASS_RECON" || myUnitDefinition.CoreClass === "CORE_CLASS_SUPPORT")) {
            this.unitFlagIcon.style.marginTop = '0.15rem';
            return;
        }
    }

    updateHostilityIndicator() {
        if (!this.unit){
            return;
        }
        const relationship = this.getRelationship();
        if (relationship == IndependentRelationship.NOT_APPLICABLE) {
            console.warn("Village Banner unable to determine affinity relationship.");
            return;
        }
        if (relationship == IndependentRelationship.FRIENDLY){
            switch (getCurrentSettings().HostilityIndicator) {
                case 'iconSwap': {
                    this.stopHostileIconSwap();
                    break;
                }
                case 'flagGlow': {
                    const classList = this.Root.classList;
                    classList.toggle('unit-flag--friendly', true);
                    classList.toggle('unit-flag--neutral', false);
                    classList.toggle('unit-flag--hostile', false);
                    break;
                }
                case 'disabled': {
                    break;
                }
            }
        }
        if (relationship == IndependentRelationship.NEUTRAL){
            switch (getCurrentSettings().HostilityIndicator) {
                case 'iconSwap': {
                    this.stopHostileIconSwap();
                    break;
                }
                case 'flagGlow': {
                    const classList = this.Root.classList;
                    classList.toggle('unit-flag--friendly', false);
                    classList.toggle('unit-flag--neutral', true);
                    classList.toggle('unit-flag--hostile', false);
                    break;
                }
                case 'disabled': {
                    break;
                }
            }
        }
        if (relationship === IndependentRelationship.HOSTILE){
            switch (getCurrentSettings().HostilityIndicator) {
                case 'iconSwap': {
                    this.startHostileIconSwap();
                    break;
                }
                case 'flagGlow': {
                    const classList = this.Root.classList;
                    classList.toggle('unit-flag--friendly', false);
                    classList.toggle('unit-flag--neutral', false);
                    classList.toggle('unit-flag--hostile', true);
                    break;
                }
                case 'disabled': {
                    break;
                }
            }
        }
    }
    updateMovement() {
        if (this.unit) {
            if (this.unit.isOnMap && this.unit.Movement) {
                this.checkUnitPosition(this.unit);
            }
        }
    }
    checkUnitPosition(unit) {
        const units = MapUnits.getUnits(unit.location.x, unit.location.y);
        const numUnits = units.length;
        const cityComponentID = GameplayMap.getOwningCityFromXY(unit.location.x, unit.location.y);
        let bannerOffset = 0;
        const constructibles = MapConstructibles.getConstructibles(unit.location.x, unit.location.y);
        constructibles.some(item => {
            const instance = Constructibles.getByComponentID(item);
            if (instance) {
                const info = GameInfo.Constructibles.lookup(instance.type);
                if (info && (info.ConstructibleType === "IMPROVEMENT_VILLAGE" || info.ConstructibleType === "IMPROVEMENT_ENCAMPMENT" || info.ConstructibleType === "BUILDING_CITY_HALL" || info.ConstructibleType === "BUILDING_PALACE")) {
                    if (cityComponentID) {
                        const city = Cities.get(cityComponentID);
                        if (city && city.isTown) {
                            bannerOffset = 24;
                            return true;
                        }
                        else {
                            bannerOffset = 24;
                            return true;
                        }
                    }
                }
            }
            return false;
        });

        if (numUnits > 0) {
            let offset = Math.round(-numUnits / 2) + 1;
            for (let i = 0; i < numUnits; i++) {
                const unitFlag = UnitFlagManager.instance.getFlag(units[i]);
                if (!unitFlag) {
                    continue;
                }
                unitFlag.updateTop(offset, bannerOffset);
                offset++;
            }
        }
    }
    updateTop(offset, bannerOffset) {
        if (this.unitContainer) {
            this.unitContainer.style.left = Layout.pixels(offset * -54);
            if (bannerOffset != 0) {
                this.unitContainer.style.top = Layout.pixels(bannerOffset);
            }
            else {
                this.unitContainer.style.top = 0;
            }
        }
    }
    
    // ------------------ MODIFIED BY NASU ------------------ END ------------------
    updateAffinity() {
        this.updateHostilityIndicator();
        this.realizeTooltip();
    }
    
    /**
     * Helper to get the affinity relationship between the player and independent power.
     * @returns {IndependentRelationship} enum representing the affinity level
     */
    getRelationship() {
        const localObserverID = GameContext.localObserverID;
        const playerID = this.componentID.owner;
        const player = Players.get(playerID);
        if (!player) {
            console.warn(`unit-flags-independent-powers: Unable to get affinity relationship due to null player from playerID ${playerID}`);
            return IndependentRelationship.NOT_APPLICABLE;
        }
        if (!player.isIndependent) {
            console.warn(`unit-flags-independent-powers: Unable to get affinity relationship due to non-independent player from playerID ${playerID}, name: ${player.name}`);
            return IndependentRelationship.NOT_APPLICABLE;
        }
        return Game.IndependentPowers.getIndependentRelationship(this.independentID, localObserverID);
    }
    get componentID() {
        return this._componentID;
    }
    get unit() {
        const unit = Units.get(this.componentID);
        if (!unit) {
            console.error("unit-flags-independent-powers: Failed attempt to get a unit for: ", ComponentID.toLogString(this.componentID));
        }
        return unit;
    }
}
Controls.define('unit-flags-independent-powers', {
    createInstance: IndependentPowersUnitFlag,
    description: 'Independent Powers Unit Flag',
    classNames: ['unit-flag', 'allowCameraMovement'],
    styles: ['fs://game/base-standard/ui/unit-flags/unit-flags.css']
});
UnitFlagFactory.registerStyle(new IndependentPowersFlagMaker());

//# sourceMappingURL=file:///base-standard/ui/unit-flags/unit-flags-independent-powers.js.map
