/**
 * @file unit-flags.ts
 * @copyright 2021-2024, Firaxis Games
 * @description Generic unit flag implementation; the default unit flag if no specific one is found.
 * The flag manages the lifetime and update of any additional 3D pieces and/or overlays.
 */
// ------------------ MODIFIED BY NASU ------------------ START ------------------
import { ComponentID } from '/core/ui/utilities/utilities-component-id.js';
import { Icon } from '/core/ui/utilities/utilities-image.js';
import { MustGetElement } from '/core/ui/utilities/utilities-dom.js';
import { UnitFlagFactory, UnitFlagManager } from '/base-standard/ui/unit-flags/unit-flag-manager.js';
import { utils } from '/core/ui/graph-layout/utils.js';
import { Layout } from '/core/ui/utilities/utilities-layout.js';
import { ActionActivateEvent } from '/core/ui/components/fxs-activatable.js';
import { DebugUtils } from '/scripts/debug-utils.js';
import { animationClock } from '/scripts/war-icon-sync-clock.js';
import UnitSelection from '/base-standard/ui/unit-selection/unit-selection.js';
import { getSettings } from '/scripts/nasuellia-alt-unit-flags-and-bars-settings.js';
const getCurrentSettings = () => getSettings("nasuelliaAltUnitFlagsAndBarsSettingsKey");

const hasPulseGlow = document.querySelector('style[data-pulse-glow]');
if (!hasPulseGlow) {
    const styleSheet = document.createElement('style');
    styleSheet.setAttribute('data-pulse-glow', 'true');
    styleSheet.textContent = `@keyframes pulseGlow {0% { transform: scale(1.00); opacity: 0.5; } 50% { transform: scale(1.15); opacity: 0.80; } 100% { transform: scale(1.00); opacity: 0.5; }}`;
    document.head.appendChild(styleSheet);
}

function manipulateColor(color, percent) {
    const rgb = color.match(/\d+/g)?.map(Number);
    if (!rgb || rgb.length < 3) {
        return color;
    }
    const darkened = rgb.map(value => Math.round(value * (1 - percent)));
    return `rgb(${darkened[0]}, ${darkened[1]}, ${darkened[2]})`;
}

// ------------------ MODIFIED BY NASU ------------------ END ------------------
class GenericFlagMaker {
    initialize() {
    }
    isMatch(unit, unitDefinition, others) {
        if (!others) { // If no others are passed in, then this will handle as a default.
            return true;
        }
        if (others.length < 1) { // empty or just filled with refernce to self
            return true;
        }
        const betterMatch = others.some((factory) => {
            if (factory === this) { // Ignore self.
                return false;
            }
            return factory.isMatch(unit, unitDefinition); // will return on first true
        });
        if (betterMatch) { // if a differnt factory said it was a match, assume it's better than this default one
            return false;
        }
        return true;
    }
    getComponentName() {
        return "unit-flag";
    }
}
export class GenericUnitFlag extends Component {
    constructor() {
        super(...arguments);
        this._componentID = ComponentID.getInvalidID();
        this._worldAnchor = null;
        this._isManagerTracked = true;
        this.engineInputListener = this.onEngineInput.bind(this);
        this.beforeUnloadListener = this.onUnload.bind(this);
        this.isHidden = false;
        // Healthbar color threshholds, sync any updates to interact-unit.ts
        this.MEDIUM_HEALTH_THRESHHOLD = .75;
        this.LOW_HEALTH_THRESHHOLD = .5;
        this.unitContainer = null;
        this.unitHealthBar = null;
        this.unitHealthBarInner = null;
        this.unitFlagIcon = null;
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.unitHealthBarContainer = null;
        this.raiseUnitSelectionListener = this.onRaiseUnitSelection.bind(this);
        this.lowerUnitSelectionListener = this.onLowerUnitSelection.bind(this);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        /**
         * A vertical offset when the unit is 'stacked' with other units.
         * TODO - The unit world anchor should be able to incorporate this offset in C++ to avoid constantly recalculating this in Script.
         */
        this.flagOffset = 0;
    }

    updateArmy() {
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        if (this.unit.isCommanderUnit) {
            this.realizePromotions();
            this.updateArmyDots();
        }
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
    }
    onAttach() {
        super.onAttach();
        this.Root.classList.add("flex", "flex-col", "justify-center", "items-center", "absolute", "opacity-100");
        const id = this.Root.getAttribute('unit-id');
        this._componentID = ComponentID.fromString(id);
        this._isManagerTracked = this.Root.getAttribute('manager-tracked') != "false";
        if (GameContext.localObserverID == this._componentID.owner || (GameContext.localObserverID == PlayerIds.OBSERVER_ID)) {
            this.Root.classList.add("cursor-pointer");
        }
        // Obtain player colors used later on.
        let playerColorPri = 'rgb(0, 0, 0)';
        let playerColorSec = 'rgb(255, 255, 255)';
        if (Players.isValid(this.componentID.owner)) {
            playerColorPri = UI.Player.getPrimaryColorValueAsString(this.componentID.owner);
            playerColorSec = UI.Player.getSecondaryColorValueAsString(this.componentID.owner);
        }
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        const unit = Units.get(this.componentID);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        // Container
        const unitFlagContainer = document.createElement('div');
        unitFlagContainer.classList.add('unit-flag__container', 'absolute', '-top-6', '-left-6', 'pointer-events-auto', 'flex', 'flex-col', 'justify-center', 'items-center', 'w-12', 'h-12');
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        unitFlagContainer.style.width = '3.75rem';
        unitFlagContainer.style.height = '3.75rem';
        unitFlagContainer.style.transition = 'transform 0.3s ease-in-out';
        this.unitContainer = unitFlagContainer;
        //this.unitContainer.style.top = '2rem';
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        // Shadow
        const unitFlagShadow = document.createElement('div');
        unitFlagShadow.classList.add('unit-flag__shadow', 'pointer-events-none', 'absolute', 'inset-0', 'bg-cover');
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        unitFlagShadow.style.filter = 'blur(14px) opacity(33%)';
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        unitFlagContainer.appendChild(unitFlagShadow);
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        // Hostility border
        const unitflagHostileOutline = document.createElement('div');
        unitflagHostileOutline.classList.add('unit-flag__hostile-outline', 'absolute', 'pointer-events-none');
        unitflagHostileOutline.style.filter = 'blur(14px)';
        unitflagHostileOutline.style.backgroundColor = playerColorSec;
        unitflagHostileOutline.style.animation = 'pulseGlow 1.5s ease-in-out infinite';
        unitFlagContainer.appendChild(unitflagHostileOutline);
        /*
        if (this.componentID.owner == GameContext.localObserverID) {
            // Vanilla Broken Highlight
            const unitFlagHighlight = document.createElement('div');
            unitFlagHighlight.classList.add('unit-flag__highlight', 'opacity-0', 'pointer-events-none', 'absolute', 'bg-no-repeat');
            unitFlagContainer.appendChild(unitFlagHighlight);
        }
         */
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        // Inner Shape
        const unitFlagInnerShape = document.createElement('div');
        unitFlagInnerShape.classList.add('unit-flag__shape', 'unit-flag__shape--inner', "pointer-events-none", "absolute", "inset-0", "bg-no-repeat");
        unitFlagInnerShape.style.fxsBackgroundImageTint = playerColorPri;
        unitFlagContainer.appendChild(unitFlagInnerShape);
        // Outer Shape
        const unitFlagOuterShape = document.createElement('div');
        unitFlagOuterShape.classList.add('unit-flag__shape', 'unit-flag__shape--outer', "pointer-events-none", "absolute", "inset-0", "bg-no-repeat");
        unitFlagOuterShape.style.fxsBackgroundImageTint = playerColorSec;
        unitFlagContainer.appendChild(unitFlagOuterShape);
        // Healthbar Container
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.unitHealthBarContainer = document.createElement('div');
        this.unitHealthBarContainer.classList.add('unit-flag__healthbar-container', 'absolute', 'h-full', 'self-center', 'pointer-events-none');
        this.unitHealthBarContainer.style.width = '2.2rem';
        this.adjustHealthbarPosition();
        unitFlagContainer.appendChild(this.unitHealthBarContainer);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        // Healthbar Outer
        const unitFlagHealthbar = document.createElement('div');
        unitFlagHealthbar.classList.add('unit-flag__healthbar', 'absolute', 'h-1', 'bg-black');
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        unitFlagHealthbar.style.height = '0.700rem';
        unitFlagHealthbar.style.border = `0.20rem solid ${playerColorPri}`;
        unitFlagHealthbar.style.backgroundColor = playerColorPri;
        unitFlagHealthbar.style.borderRadius = '0.66rem';
        unitFlagHealthbar.style.boxShadow = "0px 0px 20px rgba(0, 0, 0, 0.9)";
        this.unitHealthBarContainer.appendChild(unitFlagHealthbar);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.unitHealthBar = unitFlagHealthbar;
        const unitFlagHealthbarSizer = document.createElement('div');
        unitFlagHealthbarSizer.classList.add('unit-flag__healthbar-sizer', 'relative', 'h-3', 'w-full');
        unitFlagHealthbar.appendChild(unitFlagHealthbarSizer);
        const unitFlagHealthbarInner = document.createElement('div');
        // Healthbar Inner
        unitFlagHealthbarInner.classList.add('unit-flag__healthbar-inner', 'absolute', 'h-full', 'bg-no-repeat');
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        unitFlagHealthbarInner.style.borderRadius = '0.66rem';
        unitFlagHealthbarInner.style.backgroundColor = playerColorSec;
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        unitFlagHealthbar.appendChild(unitFlagHealthbarInner);
        this.unitHealthBarInner = unitFlagHealthbarInner;
        // Icon
        const unitFlagIcon = document.createElement('div');
        unitFlagIcon.classList.add('unit-flag__icon', 'pointer-events-none', 'absolute', 'bg-contain', 'bg-no-repeat');
        unitFlagIcon.style.fxsBackgroundImageTint = playerColorSec;
        this.unitFlagIcon = unitFlagIcon;
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        unitFlagContainer.appendChild(this.unitFlagIcon);
        this.adjustUnitIconPosition(false);
        // Promotion / Tier number
        const unitFlagPromotionNumber = document.createElement('div');
        unitFlagPromotionNumber.classList.add('unit-flag__promotion-number', 'font-body', 'text-2xs', 'absolute', 'text-center');
        unitFlagPromotionNumber.style.color = playerColorSec;
        unitFlagPromotionNumber.style.left = '50%';
        unitFlagPromotionNumber.style.top = '2.45rem';
        unitFlagPromotionNumber.style.marginLeft = '-0.55rem';
        unitFlagPromotionNumber.style.fontWeight  = "800";
        unitFlagPromotionNumber.style.fontSize = '0.55rem';
        unitFlagPromotionNumber.style.width = '1.05rem';
        unitFlagPromotionNumber.style.height = '1.05rem';
        unitFlagPromotionNumber.style.lineHeight = '1.00rem';
        unitFlagPromotionNumber.style.backgroundColor = manipulateColor(playerColorPri, 0.15);
        unitFlagPromotionNumber.style.border = `0.07rem solid ${playerColorSec}`;
        unitFlagPromotionNumber.style.borderRadius = '50%';
        unitFlagContainer.appendChild(unitFlagPromotionNumber);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        /*
        // Chevrons
        const unitFlagTierGraphic = document.createElement('div');
        unitFlagTierGraphic.classList.add('unit-flag__tier-graphic', 'absolute', 'w-4', 'h-4', 'left-4', 'right-0', 'bottom-0', 'bg-cover', 'bg-no-repeat');
        if (unit) {
            const unitDefinition = GameInfo.Units.lookup(unit.type);
            if (unitDefinition && unitDefinition.Tier) {
                unitFlagTierGraphic.style.backgroundImage = `url('fs://game/unit_chevron-0${unitDefinition.Tier}.png')`;
            }
        }
        unitFlagTierGraphic.style.width = '1.33rem';
        unitFlagTierGraphic.style.height = '1.33rem';
        unitFlagTierGraphic.style.top = '1.5rem';
        unitFlagTierGraphic.style.left = '50%';
        unitFlagTierGraphic.style.transform = 'translate(-50%, 100%)';
        unitFlagContainer.appendChild(unitFlagTierGraphic);
        // Army Stats
        const unitFlagArmyStats = document.createElement('div');
        unitFlagArmyStats.classList.add('unit-flag__army-stats"', 'items-center', 'text-center', 'absolute', '-left-3', '-right-3', 'bg-transparent');
        unitFlagContainer.appendChild(unitFlagArmyStats);
         */
        // Army Dots
        const unitFlagLeftDots = document.createElement('div');
        unitFlagLeftDots.classList.add('unit-flag__left-dots', 'absolute', 'flex', 'flex-col-reverse', 'justify-end');
        this.unitFlagLeftDots = unitFlagLeftDots;
        const unitFlagRightDots = document.createElement('div');
        unitFlagRightDots.classList.add('unit-flag__right-dots', 'absolute', 'flex', 'flex-col-reverse', 'justify-end');
        unitFlagContainer.appendChild(unitFlagRightDots);
        
        if (unit && unit.isFleetCommander || unit && unit.isSquadronCommander  || unit && unit.isAerodromeCommander) {
            unitFlagLeftDots.style.top = '0.65rem';
            unitFlagLeftDots.style.left = '0.70rem';
            unitFlagRightDots.style.top = '0.65rem';
            unitFlagRightDots.style.right = '0.70rem';
        }
        if (unit && unit.isArmyCommander) {
            unitFlagLeftDots.style.top = '0.85rem';
            unitFlagLeftDots.style.left = '0.45rem';
            unitFlagLeftDots.style.transform = 'rotate(-45deg)';
            unitFlagRightDots.style.top = '0.85rem';
            unitFlagRightDots.style.right = '0.45rem';
            unitFlagRightDots.style.transform = 'rotate(45deg)';
        }
        unitFlagContainer.appendChild(unitFlagLeftDots);
        this.unitFlagRightDots = unitFlagRightDots;
        // Status icon
        this.statusIconsContainer = document.createElement('div');
        this.statusIconsContainer.style.position = 'absolute';
        this.statusIconsContainer.style.display = 'flex';
        this.unitContainer.appendChild(this.statusIconsContainer);
        // Logic for oscillation, status flags, and promotion dots
        this.activeStatusIcons = new Map();
        this.readyAnimationLoopTimer = null;
        this.readyAnimationTracker = false;
        this.promotionAnimationLoopTimer = null;
        this.promotionAnimationTracker = false;
        this.iconSwapSubscription = null;
        this.originalUnitIcon = null;
        this.initializeArmyDots();
        this.updateMovement();
        this.updateArmyDots();
        this.updateStatusIcons();
        engine.on("UnitOperationSegmentComplete", this.onUnitOperationRefresh, this);
        engine.on("UnitOperationDeactivated", this.onUnitOperationRefresh, this);
        engine.on("UnitOperationsCleared", this.onUnitOperationRefresh, this);
        engine.on("UnitOperationAdded", this.onUnitOperationRefresh, this);
        engine.on('UnitCommandStarted', this.onUnitOperationRefresh, this);
        engine.on('DiplomacyEventStarted', this.onDiplomacyEventStarted, this);
        engine.on('DiplomacyEventEnded', this.onDiplomacyEventEnded, this);
        engine.on('TurnBegin', this.onTurnBegin, this);
        engine.on('UnitExperienceChanged', this.onUnitExperienceChanged, this);
        UnitSelection.onLower.on(this.lowerUnitSelectionListener);
        UnitSelection.onRaise.on(this.raiseUnitSelectionListener);
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        // Finalize
        this.Root.appendChild(unitFlagContainer);
        engine.on('BeforeUnload', this.beforeUnloadListener);
        if (this._isManagerTracked) {
            const manager = UnitFlagManager.instance;
            manager.addChildForTracking(this);
        }
        // Sanity check early on (subsequent unit calls will be from componentID in getter)
        if (!unit) {
            console.error("Could not attach unit flag; no unit object for cid: ", ComponentID.toLogString(this.componentID));
            return;
        }
        this.Root.addEventListener('engine-input', this.engineInputListener);
        if (this._isManagerTracked) {
            this.makeWorldAnchor(this.componentID);
        }
        if (unit.Movement) {
            this.setMovementPoints(unit.Movement.movementMovesRemaining);
        }
        this.realizeIcon();
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        this.updateHostilityIndicator();
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        this.realizeUnitHealth();
        this.realizeTooltip();
        this.realizePromotions();
        const location = unit.location;
        const revealedState = GameplayMap.getRevealedState(GameContext.localObserverID, location.x, location.y);
        if (!this.isHidden) {
            this.setVisibility(revealedState);
        }
        if (this._isManagerTracked) {
            this.checkUnitPosition(unit);
        }
    }

    // ------------------ MODIFIED BY NASU ------------------ START ------------------

    onUnitExperienceChanged(data){
        setTimeout(() => {
            this.updatePromotions();
        }, 100);
    }
    
    onTurnBegin(data) {
        setTimeout(() => {
            this.updateMovement();
            this.updateStatusIcons();
            this.realizePromotions();
            this.updateHostilityIndicator();
        }, 100);
    }
    
    onUnitOperationRefresh(eventData) {
        if (eventData.unit && eventData.unit.owner === 0) {
            if (eventData.unit.id === this.componentID.id) {
                setTimeout(() => {
                    this.updateStatusIcons();
                    this.updateMovement();
                }, 100);
            }
        }
    }
    onDiplomacyEventStarted(data) {
        const actionType = data.actionType;
        if (actionType === -434781988) {
            setTimeout(() => {
                this.updateHostilityIndicator();
            }, 100);
        }
    }
    onDiplomacyEventEnded(data) {
        const actionType = data.actionType;
        if (actionType === -434781988) {
            setTimeout(() => {
                this.updateHostilityIndicator();
            }, 100);
        }
    }

    onRaiseUnitSelection(cid) {
        if (getCurrentSettings().SelectionIndicator === "brighten") {
            if (cid.owner === this.unit.owner && cid.id === this.unit.localId) {
                this.unitContainer.style.filter = 'brightness(135%)';
            }
        }
    }
    onLowerUnitSelection(cid) {
        if (cid) {
            if (cid.owner === this.unit.owner && cid.id === this.unit.localId) {
                this.unitContainer.style.filter = 'brightness(100%)';
            }
        }
    }
    // ------------------ MODIFIED BY NASU ------------------ END ------------------
    
    onUnload() {
        this.cleanup();
    }

    // ------------------ MODIFIED BY NASU ------------------ START ------------------
    onDetach() {
        engine.off("UnitOperationSegmentComplete", this.onUnitOperationRefresh, this);
        engine.off("UnitOperationDeactivated", this.onUnitOperationRefresh, this);
        engine.off("UnitOperationsCleared", this.onUnitOperationRefresh, this);
        engine.off("UnitOperationAdded", this.onUnitOperationRefresh, this);
        engine.off('UnitCommandStarted', this.onUnitCommandStartedListener, this);
        engine.off('DiplomacyEventStarted', this.onDiplomacyEventStarted, this);
        engine.off('DiplomacyEventEnded', this.onDiplomacyEventEnded, this);
        engine.off('TurnBegin', this.onTurnBegin, this);
        engine.off('UnitExperienceChanged', this.onUnitExperienceChanged, this);
        UnitSelection.onLower.off(this.lowerUnitSelectionListener);
        UnitSelection.onRaise.off(this.raiseUnitSelectionListener);
        this.cleanup();
        super.onDetach();
    }
    // ------------------ MODIFIED BY NASU ------------------ END ------------------
    cleanup() {
        if (this._isManagerTracked) {
            const manager = UnitFlagManager.instance;
            manager.removeChildFromTracking(this);
            engine.off('BeforeUnload', this.beforeUnloadListener);
            this.destroyWorldAnchor();
        }
        this.Root.removeEventListener('engine-input', this.engineInputListener);
        this._componentID = ComponentID.getInvalidID();
    }
    onEngineInput(inputEvent) {
        if (inputEvent.detail.status != InputActionStatuses.FINISH) {
            return;
        }
        if (this.unitContainer?.classList.contains("disabled")) {
            return;
        }
        if (inputEvent.detail.name == "accept" || inputEvent.detail.name == "mousebutton-left" || inputEvent.detail.name == "touch-tap") {
            if (ComponentID.isInvalid(this._componentID)) {
                console.warn("Attempt to activate a unit-flag but invalid associated unit.");
                return;
            }
            // Has to be the owner or need to be pure observer
            if (GameContext.localObserverID != this._componentID.owner && (GameContext.localObserverID != PlayerIds.OBSERVER_ID)) {
                return;
            }
            UI.Player.selectUnit(this._componentID);
            window.dispatchEvent(new SetActivatedComponentEvent(null));
            this.Root.dispatchEvent(new ActionActivateEvent(inputEvent.detail.x, inputEvent.detail.y));
            inputEvent.stopPropagation();
            inputEvent.preventDefault();
        }
    }
    hide() {
        this.isHidden = true;
        this.unitContainer?.classList.add("hidden");
    }
    show() {
        this.isHidden = false;
        if (this.unit) {
            const location = this.unit.location;
            const revealedState = GameplayMap.getRevealedState(GameContext.localObserverID, location.x, location.y);
            this.setVisibility(revealedState);
        }
    }
    disable() {
        this.unitContainer?.classList.add("disabled");
    }
    enable() {
        this.unitContainer?.classList.remove("disabled");
    }
    realizeUnitHealth() {
        if (!this.unitHealthBar) {
            console.error("unit-flags: realizeUnitHealth(): Missing this.unitHealthBar with '.unit-flag__healthbar'. cid: " + ComponentID.toLogString(this.componentID));
            return;
        }
        let unit = this.unit;
        // reset health stats
        this.unitHealthBar.classList.toggle("unit-flag__healthbar-med-health", false);
        this.unitHealthBar.classList.toggle("unit-flag__healthbar-low-health", false);
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        let damage = 1.0;
        if (unit?.Health) {
            damage = (unit.Health.maxDamage - unit.Health.damage) / unit.Health.maxDamage;
            if (unit.Health.damage > 0 || getCurrentSettings().HealthbarVisibility === 'permanent') {
                this.unitContainer?.classList.toggle("unit-flag--with-healthbar", true);
                this.adjustStatusIconPosition(true);
            }
            else {
                this.unitContainer?.classList.toggle("unit-flag--with-healthbar", false);
                this.adjustStatusIconPosition(false);
            }
            /*
            if (damage <= this.MEDIUM_HEALTH_THRESHHOLD && damage >= this.LOW_HEALTH_THRESHHOLD) {
                this.unitContainer?.classList.toggle("unit-flag__healthbar-med-health", true);
                this.unitContainer?.classList.toggle("unit-flag__healthbar-low-health", false);
            }
            else if (damage < this.LOW_HEALTH_THRESHHOLD) {
                this.unitContainer?.classList.toggle("unit-flag__healthbar-med-health", false);
                this.unitContainer?.classList.toggle("unit-flag__healthbar-low-health", true);
            }
            else {
                this.unitContainer?.classList.toggle("unit-flag__healthbar-med-health", false);
                this.unitContainer?.classList.toggle("unit-flag__healthbar-low-health", false);
            }
             */
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
        if (this.unitHealthBarInner) {
            this.unitHealthBarInner.style.widthPERCENT = utils.clamp(damage, 0, 1) * 100;
        }
    }
    realizeIcon() {
        const unit = this.unit;
        if (!unit) {
            console.error("Unit flag finished loading its HTML content but is not associated with a valid unit. cid: ", ComponentID.toLogString(this.componentID));
            return;
        }
        const unitDefinition = GameInfo.Units.lookup(unit.type);
        if (!unitDefinition) {
            console.warn("unit-flags: Cannot set unit flag icon due to missing Unit Definition. type: ", unit.type, "  cid: ", ComponentID.toLogString(this.componentID));
            return;
        }
        if ((unit.isArmyCommander) || (unitDefinition.UnitType == "UNIT_AERODROME_COMMANDER")) {
            this.unitContainer?.classList.add('unit-flag--army');
        }
        else if (unit.Combat?.canAttack) {
            this.unitContainer?.classList.add('unit-flag--combat');
        }
        else if (unitDefinition.CoreClass != "CORE_CLASS_SUPPORT" && unitDefinition.CoreClass != "CORE_CLASS_RECON") {
            this.unitContainer?.classList.add('unit-flag--civilian');
        }
        if (this.unitFlagIcon) {
            // ------------------ MODIFIED BY NASU ------------------ START ------------------
            const iconName = Icon.getUnitIconFromDefinition(unitDefinition);
            this.originalUnitIcon = iconName;
            this.unitFlagIcon.style.backgroundImage = `url(${iconName})`;
            // ------------------ MODIFIED BY NASU ------------------ END ------------------
        }
        this.unitContainer?.classList.toggle('owned-unit', (this.componentID.owner == GameContext.localObserverID));
    }
    realizeTooltip() {
        const playerId = this.componentID.owner;
        const player = Players.get(playerId);
        if (player) {
            if (player.isMinor && player.Influence?.hasSuzerain) {
                const suzerain = player.Influence.getSuzerain();
                const suzerainPlayer = Players.get(suzerain);
                if (!suzerainPlayer) {
                    return;
                }
                const unit = this.unit;
                const unitName = ((unit) ? Locale.compose(unit.name) : "ERROR, unit: " + ComponentID.toLogString(this._componentID));
                // ------------------ MODIFIED BY NASU ------------------ START ------------------
                const suzerainedPlayerName = Locale.compose(player.civilizationFullName);
                // ------------------ MODIFIED BY NASU ------------------ END ------------------
                const tooltipDiv = this.Root.querySelector('.unit-flag__container');
                if (tooltipDiv) {
                    tooltipDiv.setAttribute("data-tooltip-content", `<div>${unitName}</div><div>${suzerainedPlayerName}</div>`);
                }
            }
            else {
                const playerName = Locale.compose(player.name);
                let unit = this.unit;
                let unitName = ((unit) ? Locale.compose(unit.name) : "ERROR, unit: " + ComponentID.toLogString(this._componentID));
                const tooltipDiv = this.Root.querySelector('.unit-flag__container');
                if (tooltipDiv) {
                    // ------------------ MODIFIED BY NASU ------------------ START ------------------
                    const playerCivName = Locale.compose(player.civilizationFullName);
                    tooltipDiv.setAttribute("data-tooltip-content", `<div>${unitName}</div><div>${playerName}</div><div>${playerCivName}</div>`);
                    // ------------------ MODIFIED BY NASU ------------------ END ------------------
                }
            }
        }
    }
    // ------------------ MODIFIED BY NASU ------------------ START ------------------
    realizePromotions() {
        const unitDefinition = GameInfo.Units.lookup(this.unit.type);
        const promotionContainer = MustGetElement(".unit-flag__promotion-number", this.Root);
        if (!this.unit || !unitDefinition) {
            console.warn("unit-flag: Cannot set promotions due to missing Unit Definition. type: ", this.unit.type, "  cid: ", ComponentID.toLogString(this.componentID));
            return;
        }
        if (this.unit.isCommanderUnit) {
            while (promotionContainer.hasChildNodes()) {
                promotionContainer.removeChild(promotionContainer.lastChild);
            }
            const level = this.unit.Experience?.getLevel;
            const promotions = this.unit.Experience.getNumPromotions;
            const levelIndicator = document.createElement("div");
            levelIndicator.classList.add("w-4", "h-4");
            levelIndicator.innerHTML = level.toString();
            promotionContainer.appendChild(levelIndicator);
            this.Root.classList.add("unit-flag--has-promotions");
            const canPromote = level > promotions;
            if (canPromote && this.unit.owner === GameContext.localObserverID) {
                this.startFlipTwitch(promotionContainer);
            }
            else {
                this.stopFlipTwitch(promotionContainer);
            }
        }
        else if (unitDefinition.CoreClass === "CORE_CLASS_MILITARY" && unitDefinition.UnitType !== "UNIT_JAGUAR_SLAYER") {
            while (promotionContainer.hasChildNodes()) {
                promotionContainer.removeChild(promotionContainer.lastChild);
            }
            const numPromotions = unitDefinition.Tier;
            const numPromotionsRoman = numPromotions > 0 && numPromotions <= 10 ? ['I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX', 'X'][numPromotions - 1] : '';
            const promotionNumber = document.createElement("div");
            promotionNumber.classList.add("w-4", "h-4");
            promotionNumber.innerHTML = numPromotionsRoman;
            promotionContainer.appendChild(promotionNumber);
            this.Root.classList.add("unit-flag--has-promotions");
            const canUpgrade = !!Game.UnitCommands?.canStart(this.unit.id, "UNITCOMMAND_UPGRADE", { X: -9999, Y: -9999, UnitAbilityType: -1 }, false)?.Success;
            if (canUpgrade && this.unit.owner === GameContext.localObserverID) {
                this.startFlipTwitch(promotionContainer);
            }
            else {
                this.stopFlipTwitch(promotionContainer);
            }
        }
        else {
            promotionContainer.style.visibility = 'hidden';
        }
    }
    // ------------------ MODIFIED BY NASU ------------------ END ------------------
    
    /**
     * Change the visibility of the unit's flag.
     * @param {RevealState} state - The visibility state to change to.
     */
    setVisibility(state) {
        if (this.isHidden) {
            return;
        }
        switch (state) {
            case RevealedStates.HIDDEN:
                this.unitContainer?.classList.add("hidden");
                break;
            case RevealedStates.REVEALED:
                this.unitContainer?.classList.add("hidden");
                break;
            case RevealedStates.VISIBLE:
                this.unitContainer?.classList.remove("hidden");
                break;
            default:
                console.warn("Unknown visibility reveal type passed to unit flag. vis: ", state, "  cid: ", ComponentID.toLogString(this.componentID));
                break;
        }
    }
    makeWorldAnchor(componentID) {
        this.destroyWorldAnchor();
        // ------------------ MODIFIED BY NASU ------------------ START ------------------
        const height = 42.0;
        // ------------------ MODIFIED BY NASU ------------------ END ------------------
        const worldAnchor = WorldAnchors.RegisterUnitAnchor(componentID, height);
        if (worldAnchor !== null && worldAnchor >= 0) {
            this.Root.setAttribute('data-bind-style-transform2d', `{{UnitAnchors.offsetTransforms[${worldAnchor}].value}}`);
            this.Root.setAttribute('data-bind-style-opacity', `{{UnitAnchors.visibleValues[${worldAnchor}]}}`);
        }
        else {
            console.error(`Failed to create WorldAnchor for unit`, componentID);
        }
    }
    destroyWorldAnchor() {
        if (this._worldAnchor) {
            this.Root.removeAttribute('data-bind-style-transform2d');
            this.Root.removeAttribute('data-bind-style-opacity');
            WorldAnchors.UnregisterUnitAnchor(this._worldAnchor);
        }
        this._worldAnchor = null;
    }
    updateHealth() {
        this.realizeUnitHealth();
    }
    // ------------------ MODIFIED BY NASU ------------------ START ------------------
    initializeArmyDots() {
        this.leftDots = [];
        this.rightDots = [];
        
        const playerColorSec = UI.Player.getSecondaryColorValueAsString(this.componentID.owner);
        const playerColorPri = UI.Player.getPrimaryColorValueAsString(this.componentID.owner);
        
        for (let i = 0; i < 4; i++) {
            const dot = document.createElement('div');
            dot.classList.add('unit-flag__dot');
            dot.style.width = '0.45rem';
            dot.style.height = '0.45rem';
            dot.style.borderRadius = '50%';
            dot.style.border = `1px solid ${playerColorPri}`;
            dot.style.backgroundColor = playerColorSec;
            dot.style.marginTop = '0.05rem';
            dot.style.visibility = 'hidden';
            this.unitFlagLeftDots.appendChild(dot);
            this.leftDots.push(dot);
        }
        
        for (let i = 0; i < 4; i++) {
            const dot = document.createElement('div');
            dot.classList.add('unit-flag__dot');
            dot.style.width = '0.45rem';
            dot.style.height = '0.45rem';
            dot.style.borderRadius = '50%';
            dot.style.border = `1px solid ${playerColorPri}`;
            dot.style.backgroundColor = playerColorSec;
            dot.style.marginTop = '0.05rem';
            dot.style.visibility = 'hidden';
            this.unitFlagRightDots.appendChild(dot);
            this.rightDots.push(dot);
        }
    }
    updateArmyDots() {
        let unitCount = 0;
        if (this.unit.isCommanderUnit) {
            const army = this.unit.armyId ? Armies.get(this.unit.armyId) : null;
            unitCount = army.unitCount;
        }
        
        unitCount = Math.min(Math.max(unitCount, 0), 8);
        
        this.leftDots.forEach(dot => dot.style.visibility = 'hidden');
        this.rightDots.forEach(dot => dot.style.visibility = 'hidden');
        
        for (let i = 0; i < unitCount; i++) {
            if (i % 2 === 0) {
                const leftIndex = Math.floor(i / 2);
                if (leftIndex < 4) {
                    this.leftDots[leftIndex].style.visibility = 'visible';
                }
            } else {
                const rightIndex = Math.floor(i / 2);
                if (rightIndex < 4) {
                    this.rightDots[rightIndex].style.visibility = 'visible';
                }
            }
        }
    }
    setMovementPoints(amount) {
        const unitName = this.unit.name || 'Unknown name';
        const unitId = this.unit.localId !== undefined ? this.unit.localId : 'No localId';
        const unitOwner = this.unit.owner !== undefined ? this.unit.owner : 'No owner';
        
        if (this.unit.owner == GameContext.localObserverID) {
            if (this.unitContainer) {
                const hasMoves = amount > 0;
                const isFree = !this.unit.hasPendingOperations && this.unit.activityType === 1797587246;
                let shouldBeHighlighted = null;
                switch (getCurrentSettings().ReadyIndicatorLogic) {
                    case 'movementAndOperations': {
                        shouldBeHighlighted = hasMoves && isFree;
                        break;
                    }
                    case 'strictlyMovement': {
                        shouldBeHighlighted = hasMoves;
                        break;
                    }
                    case 'disabled': {
                        shouldBeHighlighted = false;
                        return;
                    }
                }
                
                if (getCurrentSettings().ReadyIndicatorReversion === 'vanilla') {
                    shouldBeHighlighted = !shouldBeHighlighted;
                }
               
                if (shouldBeHighlighted) {
                    switch (getCurrentSettings().ReadyIndicatorVisuals) {
                        case "bob": {
                            this.startBob();
                            break;
                        }
                        case "rock": {
                            this.startOscillation();
                            break;
                        }
                        case "pulse": {
                            this.startPulse();
                            break;
                        }
                        case "alpha": {
                            this.unitContainer.classList.toggle('no_movement', true);
                        }
                    }
                } else {
                    switch (getCurrentSettings().ReadyIndicatorVisuals) {
                        case "bob": {
                            this.stopBob();
                            break;
                        }
                        case "rock": {
                            this.stopOscillation();
                            break;
                        }
                        case "pulse": {
                            this.stopPulse();
                            break;
                        }
                        case "alpha": {
                            this.unitContainer.classList.toggle('no_movement', false);
                            break;
                        }
                    }
                }
            } else {
                console.error("unit-flags: setMovementPoints(): Missing unitContainer with '.unit-flag__container' for cid: " + ComponentID.toLogString(this.componentID));
            }
        }
    }

    startOscillation() {
        if (this.readyAnimationTracker && this.readyAnimationLoopTimer !== null) {
            clearTimeout(this.readyAnimationLoopTimer);
            this.unitContainer.style.transform = 'rotate(0deg)';
        }
        const oscillateTilt = () => {
            this.unitContainer.style.transform = 'rotate(4deg)';
            this.readyAnimationLoopTimer = setTimeout(() => {
                this.unitContainer.style.transform = 'rotate(-4deg)';
                this.readyAnimationLoopTimer = setTimeout(oscillateTilt, 300);
            }, 300);
        };
        this.readyAnimationTracker = true;
        oscillateTilt();
    }

    stopOscillation() {
        clearTimeout(this.readyAnimationLoopTimer);
        this.readyAnimationLoopTimer = null;
        this.readyAnimationTracker = false;
        this.unitContainer.style.transform = 'rotate(0deg)';
    }

    startPulse() {
        if (this.readyAnimationTracker && this.readyAnimationLoopTimer !== null) {
            clearTimeout(this.readyAnimationLoopTimer);
            this.unitContainer.style.transform = 'scale(1)';
        }
        this.unitContainer.style.transition = 'transform 0.5s ease-in-out';
        let growing = true;
        const pulse = () => {
            this.unitContainer.style.transform = growing ? 'scale(1.05)' : 'scale(0.95)';
            growing = !growing;
            this.readyAnimationLoopTimer = setTimeout(pulse, 500);
        };
        this.readyAnimationTracker = true;
        pulse();
    }

    stopPulse() {
        clearTimeout(this.readyAnimationLoopTimer);
        this.readyAnimationLoopTimer = null;
        this.readyAnimationTracker = false;
        this.unitContainer.style.transition = '';
        this.unitContainer.style.transform = 'scale(1)';
    }

    startBob() {
        if (this.readyAnimationTracker && this.readyAnimationLoopTimer !== null) {
            clearTimeout(this.readyAnimationLoopTimer);
            this.unitContainer.style.transform = 'translateY(0rem)';
        }
        this.unitContainer.style.transition = 'transform 0.5s ease-in-out';
        let up = true;
        const bob = () => {
            this.unitContainer.style.transform = up ? 'translateY(-0.10rem)' : 'translateY(0.10rem)';
            up = !up;
            this.readyAnimationLoopTimer = setTimeout(bob, 500);
        };
        this.readyAnimationTracker = true;
        bob();
    }

    stopBob() {
        clearTimeout(this.readyAnimationLoopTimer);
        this.readyAnimationLoopTimer = null;
        this.readyAnimationTracker = false;
        this.unitContainer.style.transition = '';
        this.unitContainer.style.transform = 'translateY(0rem)';
    }

    startFlipTwitch(view) {
        if (this.promotionAnimationTracker && this.promotionAnimationLoopTimer !== null) {
            clearTimeout(this.promotionAnimationLoopTimer);
            view.style.transform = 'rotate(0deg)';
        };

        const doFlip = () => {
            if (!this.promotionAnimationTracker) {
                view.style.transform = 'scaleX(1)';
                return;
            }

            view.style.transition = 'transform 0.33s ease-out';
            view.style.transform = 'scaleX(-1)';

            setTimeout(() => {
                view.style.transform = 'scaleX(1)';
            }, 333);

            this.promotionAnimationLoopTimer = setTimeout(doFlip, 3000);
        };

        this.promotionAnimationTracker = true;
        doFlip();
    }

    stopFlipTwitch(view) {
        clearTimeout(this.promotionAnimationLoopTimer);
        this.promotionAnimationLoopTimer = null;
        this.promotionAnimationTracker = false;
        view.style.transition = '';
        view.style.transform = 'scaleX(1)';
    }
    
    adjustStatusIconPosition(isHealthbarOn) {
        const unit = Units.get(this.componentID);
        if (!unit) {
            return;
        }
        const myUnitDefinition = GameInfo.Units.lookup(unit.type);
        if (isHealthbarOn === true) {
            if (unit.isFleetCommander || unit.isSquadronCommander) {
                this.statusIconsContainer.style.top = '-1.15rem';
                return;
            }
            if (unit.isArmyCommander) {
                this.statusIconsContainer.style.top = '-1.50rem';
                return;
            }
            if (myUnitDefinition && myUnitDefinition.CoreClass === "CORE_CLASS_MILITARY") {
                this.statusIconsContainer.style.top = '-1.50rem';
                return;
            }
            if (myUnitDefinition && (myUnitDefinition.CoreClass === "CORE_CLASS_RECON" || myUnitDefinition.CoreClass === "CORE_CLASS_SUPPORT")) {
                this.statusIconsContainer.style.top = '-1.25rem';
                return;
            }
            this.statusIconsContainer.style.top = '-1.15rem';
        }
        else {
            if (unit.isFleetCommander || unit.isSquadronCommander) {
                this.statusIconsContainer.style.top = '-0.35rem';
                return;
            }
            if (unit.isArmyCommander) {
                this.statusIconsContainer.style.top = '-0.70rem';
                return;
            }
            if (myUnitDefinition && myUnitDefinition.CoreClass === "CORE_CLASS_MILITARY") {
                this.statusIconsContainer.style.top = '-0.65rem';
                return;
            }
            if (myUnitDefinition && (myUnitDefinition.CoreClass === "CORE_CLASS_RECON" || myUnitDefinition.CoreClass === "CORE_CLASS_SUPPORT")) {
                this.statusIconsContainer.style.top = '-0.40rem';
                return;
            }
            this.statusIconsContainer.style.top = '-0.30rem';
        }
    }
    
    adjustUnitIconPosition(isWarIcon){
        const unit = Units.get(this.componentID);
        if (!unit){
            return;
        }
        const myUnitDefinition = GameInfo.Units.lookup(unit.type);
        if (!unit || !this.unitFlagIcon) {
            return;
        }
        if (unit.isFleetCommander || unit.isSquadronCommander) {
            this.unitFlagIcon.style.marginTop = '0.5rem';
            return;
        }
        if (unit.isArmyCommander) {
            this.unitFlagIcon.style.marginTop = '-0.175rem';
            this.unitFlagIcon.style.marginLeft = isWarIcon === true ? '0.015rem' : '0.00rem';
            return;
        }
        if (myUnitDefinition && myUnitDefinition.CoreClass === "CORE_CLASS_MILITARY") {
            this.unitFlagIcon.style.marginTop = isWarIcon === true ? '0.15rem' : '0.15rem';
            this.unitFlagIcon.style.marginLeft = isWarIcon === true ? '0.00rem' : '0.00rem';
            return;
        }
        if (myUnitDefinition && (myUnitDefinition.CoreClass === "CORE_CLASS_RECON" || myUnitDefinition.CoreClass === "CORE_CLASS_SUPPORT")) {
            this.unitFlagIcon.style.marginTop = '0.15rem';
            return;
        }
    }

    adjustHealthbarPosition(){
        const unit = Units.get(this.componentID);
        if (!unit || !this.unitHealthBarContainer) {
            return;
        }
        const myUnitDefinition = GameInfo.Units.lookup(unit.type);
        if (unit.isFleetCommander || unit.isSquadronCommander) {
            this.unitHealthBarContainer.style.bottom = '0.15rem';
            return;
        }
        if (unit.isArmyCommander) {
            this.unitHealthBarContainer.style.bottom = '0.30rem';
            return;
        }
        if (myUnitDefinition && myUnitDefinition.CoreClass === "CORE_CLASS_MILITARY") {
            this.unitHealthBarContainer.style.bottom = '0.30rem';
            return;
        }
        if (myUnitDefinition && (myUnitDefinition.CoreClass === "CORE_CLASS_RECON" || myUnitDefinition.CoreClass === "CORE_CLASS_SUPPORT")) {
            this.unitHealthBarContainer.style.bottom = '0.20rem';
            return;
        }
        this.unitHealthBarContainer.style.bottom = '0.40rem';
    }

    startHostileIconSwap() {
        if (!this.unitFlagIcon || !this.originalUnitIcon) return;
        if (this.iconSwapSubscription) return;
        this.unitFlagIcon.style.transition = 'transform 0.1s ease'; // Default for initial shrink

        var updateIcon = function() {
            var phase = animationClock.getPhase();
            if (phase === 'grow') {
                this.unitFlagIcon.style.transform = 'scale(0.8)'; // Shrink before skull
                setTimeout(function() {
                    this.unitFlagIcon.style.backgroundImage = 'url(fs://game/nasuellia-alt-unit-flags-and-bars/icons/war.png)';
                    this.adjustUnitIconPosition(true);
                    this.unitFlagIcon.style.transition = 'transform 0.1s ease-out'; // Overshoot
                    this.unitFlagIcon.style.transform = 'scale(1.2)'; // Overshoot past 1.0
                }.bind(this), 100); // Half of grow’s 200ms phase
            } else if (phase === 'skull') {
                this.unitFlagIcon.style.transition = 'transform 0.1s ease-in'; // Settle to 1.0
                this.unitFlagIcon.style.transform = 'scale(1.0)';
                this.unitFlagIcon.style.backgroundImage = 'url(fs://game/nasuellia-alt-unit-flags-and-bars/icons/war.png)';
            } else if (phase === 'shrink') {
                this.unitFlagIcon.style.transform = 'scale(0.8)'; // Shrink before normal
                setTimeout(function() {
                    this.unitFlagIcon.style.backgroundImage = 'url(' + this.originalUnitIcon + ')';
                    this.adjustUnitIconPosition(false);
                    this.unitFlagIcon.style.transition = 'transform 0.1s ease-out'; // Overshoot
                    this.unitFlagIcon.style.transform = 'scale(1.2)'; // Overshoot past 1.0
                }.bind(this), 100); // Half of shrink’s 200ms phase
            } else if (phase === 'normal') {
                this.unitFlagIcon.style.transition = 'transform 0.1s ease-in'; // Settle to 1.0
                this.unitFlagIcon.style.transform = 'scale(1.0)';
                this.unitFlagIcon.style.backgroundImage = 'url(' + this.originalUnitIcon + ')';
            }
        }.bind(this);
        this.iconSwapSubscription = updateIcon;
        animationClock.subscribe(updateIcon);
        animationClock.start();
    }

    stopHostileIconSwap() {
        if (this.iconSwapSubscription) {
            animationClock.unsubscribe(this.iconSwapSubscription);
            this.iconSwapSubscription = null;
            this.unitFlagIcon.style.transition = '';
            this.unitFlagIcon.style.transform = 'scale(1.0)';
            this.unitFlagIcon.style.backgroundImage = 'url(' + this.originalUnitIcon + ')';
            this.adjustUnitIconPosition(false);
            console.warn('NASU: Stopped hostile swap, reset to normal');
        }
    }
    
    updateHostilityIndicator() {
        if (!this.unit){
            return;
        }
        const unitOwner = this.unit.owner ;
        if (!unitOwner){
            return;
        }
        let unitOwnerPlayer = Players.get(unitOwner);
        if (unitOwnerPlayer) {
            const classList = this.Root.classList;
            if (unitOwnerPlayer.Diplomacy.isAtWarWith(GameContext.localPlayerID)) {
                switch (getCurrentSettings().HostilityIndicator) {
                    case 'iconSwap': {
                        this.startHostileIconSwap();
                        break;
                    }
                    case 'flagGlow': {
                        classList.toggle('unit-flag--hostile', true);
                        break;
                    }
                    case 'disabled': {
                        break;
                    }
                }
            }
            else {
                switch (getCurrentSettings().HostilityIndicator) {
                    case 'iconSwap': {
                        this.stopHostileIconSwap();
                        break;
                    }
                    case 'flagGlow': {
                        classList.toggle('unit-flag--hostile', false);
                        break;
                    }
                    case 'disabled': {
                        break;
                    }
                }
            }
        }
    }
    
    updateStatusIcons() {
        const activityType = this.unit.activityType !== undefined ? this.unit.activityType : 'No activityType';
        const isCommanderUnit = this.unit.isCommanderUnit !== undefined ? this.unit.isCommanderUnit : 'No isCommanderUnit';
        const isBusy = this.unit.isBusy !== undefined ? this.unit.isBusy : 'No isBusy';
        const canMove = this.unit.canMove !== undefined ? this.unit.canMove : 'No canMove';
        const operationTimer = this.unit.operationTimer !== undefined ? this.unit.operationTimer : 'No operationTimer';
        const canFort = Game.UnitOperations?.canStart(this.componentID, 'UNITOPERATION_FORTIFY', { X: -9999, Y: -9999 }, true)?.Success ?? false;
        const myUnitDefinition = GameInfo.Units.lookup(this.unit.type);
        const isMilitary = myUnitDefinition.CoreClass == "CORE_CLASS_MILITARY";
        const isNaval = myUnitDefinition.FormationClass == "FORMATION_CLASS_NAVAL";
        const isScout = myUnitDefinition.UnitType == "UNIT_SCOUT";
        const isExplorer = myUnitDefinition.UnitType == "UNIT_EXPLORER";

        if (this.unit.owner === GameContext.localObserverID) {
            const activityHashes = {
                'UNITOPERATION_REST_UNTIL_HEALED': -1298237662,
                'UNITOPERATION_ALERT': -793848223,
                'UNITOPERATION_SLEEP': 38434027,
                'UNITOPERATION_FORTIFY': 1225574625,
                'UNITOPERATION_DIG': 1225574625,
                'UNITOPERATION_LOOKOUT': 1225574625,
                'UNITOPERATION_MOVING': 1225574625,
                'UNITOPERATION_SKIP_TURN': 75236491
            };
            
            const condHeal = activityType === activityHashes['UNITOPERATION_REST_UNTIL_HEALED'];
            const condAlert = activityType === activityHashes['UNITOPERATION_ALERT'];
            const condSleep = activityType === activityHashes['UNITOPERATION_SLEEP'];
            const condFort = activityType === activityHashes['UNITOPERATION_FORTIFY'] && !canMove && !isNaval && !canFort && !isBusy && (isMilitary | isCommanderUnit);
            const condDig = activityType === activityHashes['UNITOPERATION_MOVING'] && !canMove && !isNaval && !canFort && !isBusy &&isExplorer;
            const condLookout = activityType === activityHashes['UNITOPERATION_MOVING'] && !canMove && !isNaval && !canFort && !isBusy && isScout;
            const condMov = activityType === activityHashes['UNITOPERATION_MOVING'] && !canMove && isBusy && operationTimer === 1;
            const condSkip = activityType === activityHashes['UNITOPERATION_SKIP_TURN'];

            const statuses = [
                { type: 'UNITOPERATION_REST_UNTIL_HEALED', icon: 'fs://game/nasuellia-alt-unit-flags-and-bars/icons/heal.png', isActive: condHeal},
                { type: 'UNITOPERATION_ALERT', icon: 'fs://game/nasuellia-alt-unit-flags-and-bars/icons/wake.png', isActive: condAlert},
                { type: 'UNITOPERATION_SLEEP', icon: 'fs://game/nasuellia-alt-unit-flags-and-bars/icons/sleep.png', isActive: condSleep},
                { type: 'UNITOPERATION_FORTIFY', icon: 'fs://game/nasuellia-alt-unit-flags-and-bars/icons/fort.png', isActive: condFort},
                { type: 'UNITOPERATION_DIG', icon: 'fs://game/nasuellia-alt-unit-flags-and-bars/icons/dig.png', isActive: condDig && !condFort},
                { type: 'UNITOPERATION_LOOKOUT', icon: 'fs://game/nasuellia-alt-unit-flags-and-bars/icons/lookout.png', isActive: condLookout && !condFort && !condDig},
                { type: 'UNITOPERATION_MOVING', icon: 'fs://game/nasuellia-alt-unit-flags-and-bars/icons/move.png', isActive: condMov && !condFort && !condDig && !condLookout},
                { type: 'UNITOPERATION_SKIP_TURN', icon: 'fs://game/nasuellia-alt-unit-flags-and-bars/icons/skip.png', isActive: condSkip}
            ];

            const playerColorPri = UI.Player.getPrimaryColorValueAsString(this.componentID.owner);
            const playerColorSec = UI.Player.getSecondaryColorValueAsString(this.componentID.owner);

            statuses.forEach(status => {
                const existingIcon = this.activeStatusIcons.get(status.type);
                if (status.isActive && !existingIcon) {
                    const iconElement = document.createElement('div');
                    iconElement.style.width = '1.10rem';
                    iconElement.style.height = '1.10rem';
                    iconElement.style.backgroundImage = `url("${status.icon}")`;
                    iconElement.style.backgroundSize = 'contain';
                    iconElement.style.backgroundRepeat = 'no-repeat';
                    iconElement.style.fxsBackgroundImageTint = playerColorSec;
                    iconElement.style.backgroundColor = playerColorPri;
                    iconElement.style.borderRadius = '33%';
                    iconElement.style.border = `0.05rem solid ${playerColorSec}`;
                    iconElement.style.boxShadow = "0px 0px 20px rgba(0, 0, 0, 0.9)";
                    this.statusIconsContainer.appendChild(iconElement);
                    this.activeStatusIcons.set(status.type, iconElement);
                } else if (!status.isActive && existingIcon) {
                    this.statusIconsContainer.removeChild(existingIcon);
                    this.activeStatusIcons.delete(status.type);
                }
            });
        }
    }
    updateMovement() {
        if (this.unit) {
            if (this.unit.isOnMap && this.unit.Movement) {
                this.setMovementPoints(this.unit.Movement.movementMovesRemaining);
                this.checkUnitPosition(this.unit);
            }
        }
    }
    checkUnitPosition(unit) {
        const units = MapUnits.getUnits(unit.location.x, unit.location.y);
        const numUnits = units.length;
        const cityComponentID = GameplayMap.getOwningCityFromXY(unit.location.x, unit.location.y);
        let bannerOffset = 0;
        const constructibles = MapConstructibles.getConstructibles(unit.location.x, unit.location.y);
        constructibles.some(item => {
            const instance = Constructibles.getByComponentID(item);
            if (instance) {
                const info = GameInfo.Constructibles.lookup(instance.type);
                if (info && (info.ConstructibleType === "IMPROVEMENT_VILLAGE" || info.ConstructibleType === "IMPROVEMENT_ENCAMPMENT" || info.ConstructibleType === "BUILDING_CITY_HALL" || info.ConstructibleType === "BUILDING_PALACE")) {
                    if (cityComponentID) {
                        const city = Cities.get(cityComponentID);
                        if (city && city.isTown) {
                            bannerOffset = 24;
                            return true;
                        }
                        else {
                            bannerOffset = 24;
                            return true;
                        }
                    }
                }
            }
            return false;
        });
        
        if (numUnits > 0) {
            let offset = Math.round(-numUnits / 2) + 1;
            for (let i = 0; i < numUnits; i++) {
                const unitFlag = UnitFlagManager.instance.getFlag(units[i]);
                if (!unitFlag) {
                    continue;
                }
                unitFlag.updateTop(offset, bannerOffset);
                offset++;
            }
        }
    }
    updateTop(offset, bannerOffset) {
        if (this.unitContainer) {
            this.unitContainer.style.left = Layout.pixels(offset * -54);
            if (bannerOffset != 0) {
                this.unitContainer.style.top = Layout.pixels(bannerOffset);
            }
            else {
                this.unitContainer.style.top = 0;
            }
        }
    }
    
    // ------------------ MODIFIED BY NASU ------------------ END ------------------
    updatePromotions() {
        this.realizePromotions();
    }
    get componentID() {
        return this._componentID;
    }
    get unit() {
        const unit = Units.get(this.componentID);
        if (!unit) {
            console.error("Failed attempt to get a unit for unit flag: ", ComponentID.toLogString(this.componentID));
        }
        return unit;
    }
}
Controls.define('unit-flag', {
    createInstance: GenericUnitFlag,
    description: 'Unit Flag',
    classNames: ['unit-flag', 'allowCameraMovement'],
    styles: ['fs://game/base-standard/ui/unit-flags/unit-flags.css']
});
UnitFlagFactory.registerStyle(new GenericFlagMaker());
//# sourceMappingURL=file:///base-standard/ui/unit-flags/unit-flags.js.map
